#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017 Angel Luis Escribano <alescribano@gmail.com>
################################################################
# WHAT IS THIS FOR?
#   - Shows Weather (current and forecasts) from Weather.com
#   - Latest News from RTVE or BBC 
# WARNING: Should you want other sources, you will have to adapt/struggle with the APIs
################################################################
# Based on the work of many people. Thank you for sharing!!!
#    WEATHER: Jim Kemp < https://www.instructableS.com/id/Raspberry-Pi-Internet-Weather-Station/ >
#    ICONS: VClouds http://vcloudS.deviantart.com/ 
#    BACKGROUNDS: Dr. Toxic and Timdog82001
#    NEWS TICKER (ScrollText): Gunny26 < https://github.com/gunny26/pygame/blob/master/ScrollText.py >
#    SCREEN dimMER: Thobias Thelen < http://www.pygame.org/pcr/screen_dimmer/index.php >
#    TFT GPIO SCREENS: wborelli < https://www.instructableS.com/id/Raspberry-Pi-Internet-Weather-Station/ > (NOT Tested)
################################################################
### BEGIN LICENSE
# See README.TXT File
### END LICENSE
################################################################

__version__ = "1.6.1"

import pygame
import time
import string
import calendar
import settings as S
import disputil
import wtranslat

###############################################################################

class MyDisplay:
    screen = None;
    
    ####################################################################
    def __init__(self):

        if S.dispSize != None:
            # Change S.dispSize to try other resolutions (tested: 1920x1080, 1024x600, 800x600)
            # If S.dispSize matches screen size, S.it will turn into fullscreen mode and mouse will be hided
            self.screen, self.xmax, self.ymax = disputil.InitDisplay( S.dispSize[0], S.dispSize[1], windowed=True )
        else:
            self.screen, self.xmax, self.ymax = disputil.InitDisplay()
        self.xmin = self.ymin = 0
        self.xmargin = self.xmax*0.01
        self.ymargin = self.ymax*0.01

        self.fn = disputil.InitFonts( S.font )

        if self.xmax > 800:
            # Large Display
            self.dateTh = int(self.ymax*S.dTh)            # Date Text Height
            self.tmTh2 = int(self.ymax*S.tmTh)*2          # Time Text Height (Hours:Minutes)
            self.tmSTh2 = self.tmTh2/3                    # Seconds Text Height
            self.byTh = int(self.ymax*S.sTh)              # "by" text Height
        else: 
            # Small Display - NOT FULLY TESTED!!!!
            self.dateTh = int(self.ymax*S.dThS)           # Date Text Height
            self.tmTh2 = int(self.ymax*S.tmThS)*2         # Time Text Height (Hours:Minutes)
            self.tmSTh2 = self.tmTh2/3                    # Seconds Text Height
            self.byTh = int(self.ymax*S.sThS)             # "by" text Height
        self.tmdateYPos2 = self.ymax*S.clockYPos          # Clock position
        self.calYPos = self.ymax*S.calYPos                # Calendar position

    ####################################################################
    def __del__(self):
        "Destructor to make sure pygame shuts down, etc."

    ####################################################################
    def DisplayOnlyTime( self, firstRun=False ):

        xgap = self.xmargin*3
        ygap = self.ymargin*3
        columnWidth = 3

        tm2 = time.strftime( "%H:%M", time.localtime() )                                # Hour:Minutes
        tm3 = time.strftime( "%S", time.localtime() )                                   # Seconds
        (tx2,ty2) = disputil.DrawText( self.screen, tm2, self.fn, self.tmTh2, 0, S.clockc, blit=False )
        (tx3,ty3) = disputil.DrawText( self.screen, tm3, self.fn, self.tmSTh2, 1, S.clockc, blit=False )
 
        if ( int(tm2[:2])+int(tm2[-2:])+int(tm3) == 0 ) or firstRun:
            print "All"

            self.screen.fill( S.clockBkg )

            tm0 = "by alef"
            tm1a = wtranslat.elimina_tilde( time.strftime( "%A", time.localtime() ) )   # Day of week
            tm1b = time.strftime( "%B %d", time.localtime() )                           # Date
            tm4 = time.strftime( "%Y", time.localtime() )                               # Year
            century = wtranslat.TranslateT( S.lang, 107 )
            tm5 = "  %s XXI" % century                                                  # Century
            c = calendar.TextCalendar()                                                 # Calendar
            year = int( tm4 )
            month = int( time.strftime("%m", time.localtime()) )
            day = int(time.strftime( "%d", time.localtime()) )
            cal = c.formatmonth( year, month, w=columnWidth ).splitlines()              # Calendar splitted into separate lines

            (tx0,ty0) = disputil.DrawText( self.screen, tm0, self.fn, self.byTh, 0, S.loc, blit=False )
            (tx1a,ty1a) = disputil.DrawText( self.screen, tm1a, self.fn, self.dateTh/2, 0, S.clockc, blit=False )
            (tx1b,ty1b) = disputil.DrawText( self.screen, tm1b, self.fn, self.dateTh, 0, S.clockc, blit=False )
            (tx4,ty4) = disputil.DrawText( self.screen, tm4, self.fn, int(self.dateTh*1.5), 1, S.clockc, blit=False )
            (tx5,ty5) = disputil.DrawText( self.screen, tm5, self.fn, self.dateTh, 0, S.clockc, blit=False )

            # Calendar
            self.fn = disputil.InitFonts( S.calfont )                                   # Using a "mono" non-proportional font
        
            if S.lang == 'es': cal[1] = wtranslat.TranslateT( S.lang, 108 )             # Setting spanish header (troubles with accents)
            (tx,ty) = disputil.DrawText( self.screen, cal[1], self.fn, self.dateTh/2, 0, S.clockc, blit=False )
            x = ix = self.xmax - tx - xgap
            y = iy = self.calYPos + ty1a + ty1b + ygap
        
            # Draw calendar
            for i in range (1,len(cal)):
                if i == 1: color = S.clockh
                else: color = S.clockc
                disputil.DrawText( self.screen, cal[i], self.fn, self.dateTh/2, 0, color, blit=True, x=x, y=y)
                y = y + ty + ygap
            
            # Highlight current day
            (leapDay,daysTotal) = calendar.monthrange( year, month )
            # Find column
            column = day % 7 + leapDay
            if column > 7: column = column - 7
            # Find row
            row = ((day + leapDay) / 7)
            if (day + leapDay) % 7 != 0: row = row + 1
            # Highlight
            rx = ix + ( column * tx / 7 ) - xgap - columnWidth + column
            ry = iy + ( row * ( ty + ygap ) ) - columnWidth
            w = h = ty + columnWidth * 3
            rectH = (rx,ry,w,h)
            pygame.draw.rect( self.screen, S.vlightBlue, rectH )
            if day < 10: tday = " " + str(day)
            else: tday = str(day)
            disputil.DrawText( self.screen, tday, self.fn, self.dateTh/2, 1, S.clockBkg, blit=True, x=rx+columnWidth, y=ry+columnWidth*1.7 )

            self.fn = disputil.InitFonts( S.font )                                      # Coming back to "general" font

            # Print rest of values: time and date
            disputil.DrawText( self.screen, tm0, self.fn, self.byTh, 0, S.loc, blit=True, x=xgap, y=ygap )
            disputil.DrawText( self.screen, tm1a, self.fn, self.dateTh/2, 0, S.clockc, blit=True, x=x, y=self.calYPos )
            disputil.DrawText( self.screen, tm1b, self.fn, self.dateTh, 0, S.clockc, blit=True, x=x, y=self.calYPos+ty1a )
            disputil.DrawText( self.screen, tm2, self.fn, self.tmTh2, 0, S.clockc, blit=True, x=xgap, y=self.tmdateYPos2 )
            disputil.DrawText( self.screen, tm3, self.fn, self.tmSTh2, 1, S.clockc, blit=True, x=xgap+tx2, y=self.tmdateYPos2+ty3*0.36)
            disputil.DrawText( self.screen, tm4, self.fn, int(self.dateTh*1.5), 1, S.clockc, blit=True, x=(self.xmax/2)-((tx4+tx5)/2), y=self.ymax-ty4-ygap )
            disputil.DrawText( self.screen, tm5, self.fn, self.dateTh, 0, S.clockc, blit=True, x=(self.xmax/2)-((tx4+tx5)/2)+tx4, y=self.ymax-ty5*1.1-ygap )

            pygame.display.update()
        else:
            if  int(tm3) == 0:
                rect = ( xgap, self.tmdateYPos2, tx2+tx3, ty2 )
                pygame.draw.rect( self.screen, S.clockBkg, rect )
                disputil.DrawText( self.screen, tm2, self.fn, self.tmTh2, 0, S.clockc, blit=True, x=xgap, y=self.tmdateYPos2 )
                disputil.DrawText( self.screen, tm3, self.fn, self.tmSTh2, 1, S.clockc, blit=True, x=xgap+tx2, y=self.tmdateYPos2+ty3*0.36)
            else:
                rect = ( xgap+tx2, self.tmdateYPos2+ty3*0.3, tx3, ty3 )
                pygame.draw.rect( self.screen, S.clockBkg, rect )
                disputil.DrawText( self.screen, tm3, self.fn, self.tmSTh2, 1, S.clockc, blit=True, x=xgap+tx2, y=self.tmdateYPos2+ty3*0.36)

            pygame.display.update( rect )
 
        return

####################################################################

myDisp = MyDisplay()

myDisp.DisplayOnlyTime( firstRun =True )

t = None            # Seconds control to update time, just once per second
running = True

while running:

    s = int( time.strftime( "%S", time.localtime() ) )

    if ( t != s ):
        t = s
        myDisp.DisplayOnlyTime()

    disputil.event_loop()

    # Reduce CPU consumption. Higher values produce unexpected results
    time.sleep( 1 )
