#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import platform
import pygame

####################################################################
def InitDisplay( x=None, y=None , windowed=False, hideMouse=True, clearScreen=False, icon=None, caption=None ):
	"Initializes a new pygame screen using the framebuffer"
	# Based on "Python GUI in Linux frame buffer"
	# http://www.karoltomala.com/blog/?p=679
	disp_no = os.getenv("DISPLAY")
	if disp_no:
		print "X Display = {0}".format(disp_no),

	# Check which frame buffer drivers are available
	dispDriver = platform.platform()
	if "Windows" in dispDriver:
		drivers = ['windib', 'directx']
	elif "Linux" in dispDriver:
		if "arm" in dispDriver:
			# Raspberry Pi. Start with fbcon since directfb hangs with composite output
			drivers = ['fbcon', 'directfb', 'svgalib']
		else:
			drivers = ['x11','dga', 'fbcon', 'directfb', 'ggi', 'vgl', 'svgalib', 'aalib']
	else: drivers = ['x11','dga', 'fbcon', 'directfb', 'ggi', 'vgl', 'svgalib', 'aalib', 'windib', 'directx']
	found = False
	for driver in drivers:
		# Make sure that SDL_VIDEODRIVER is set
		if not os.getenv('SDL_VIDEODRIVER'):
			os.putenv('SDL_VIDEODRIVER', driver)
		try:
			pygame.display.init()
		except pygame.error:
			print 'Driver: {0} failed.'.format(driver)
			continue
		found = True
		print " - ", driver
		break

	if not found:
		raise Exception('No suitable video driver found!')
	
	size = (pygame.display.Info().current_w, pygame.display.Info().current_h)
	print "Framebuffer Size: %d x %d" % (size[0], size[1])

	if ( ( x != None) and ( y != None ) ):
		if  ( ( x == size[0] ) and ( y == size[1] ) ):
			windowed = False
		else:
			size = (x, y)

        if icon != None:
                try:
                        iconW = pygame.image.load( icon )
                        pygame.display.set_icon( iconW )
                except: print "Main icon not found. Check Settings!"

        if caption != None:
                pygame.display.set_caption( caption )

	if windowed: 
		os.environ['SDL_VIDEO_WINDOW_POS'] = "%d,%d" % (100,100)
		screen = pygame.display.set_mode( size )
		hideMouse = False
	else:
		screen = pygame.display.set_mode( size, pygame.FULLSCREEN )

	if hideMouse:
		pygame.mouse.set_visible(0)

	if clearScreen:
		screen.fill((0, 0, 0))        
		pygame.display.update()

	return screen, size[0], size[1]

####################################################################
def InitFonts( font ):
	# Initialise font support
	pygame.font.init()
	# Check if font in settings is available, or print available fonts
	if font in pygame.font.get_fonts():
		return font
	else:
		print "Font not available! Falling back to freesans"
		print "Available Fonts: ", pygame.font.get_fonts()
		return "freesans"

####################################################################
def event_loop():

    # Look for and process keyboard events to stop program
    for event in pygame.event.get():
        if event.type == pygame.KEYDOWN:
            if ( event.key == pygame.K_q ):
                # On 'q' pressed, quit the program.
                exit()
            else:
                # Otherwise, return the pressed key
                return pygame.key.name(event.key)
        if event.type == pygame.QUIT:
            # On windowed mode, exit when clicking the "x" (close window)
            exit()

####################################################################
def dim( screen, darken_factor=64, color_filter=(0,0,0), rect=None, updateD=False ):

	if rect != None:
		(x, y, w, h ) = rect
		w = int( w )
		h = int( h )
		darken = pygame.Surface( (w, h) )
	else:
		x, y = 0,0
		darken = pygame.Surface( screen.get_size() )
	darken.fill( color_filter )
	darken.set_alpha( darken_factor )
	screen.blit( darken, (x,y) )
	if updateD:  
		if rect != None: pygame.display.update( rect )
		else: pygame.display.update()

	return

####################################################################
def DrawText( screen, text, font, fsize, fbold, fcolor, blit=False, x=0, y=0, outline=None, owidth=0, ocolor=(64,64,64), oalpha=64 ):

	font = pygame.font.SysFont( font, fsize, fbold)
	rtext = font.render( text, True, fcolor )
	(rtx,rty) = rtext.get_size()

	if blit:

                if outline == "outline":
                        otext = font.render( text, True, ocolor )
                        otext.set_alpha( oalpha / 8 )				# Worked once, but now it doesn't... don't know why
                        screen.blit( otext, (x-owidth, y-owidth) )
                        screen.blit( otext, (x-owidth, y+owidth) )
                        screen.blit( otext, (x+owidth, y-owidth) )
                        screen.blit( otext, (x+owidth, y+owidth) )
                        # These next four blits are not strictly necessary, though look better (comment to improve CPU usage)
                        screen.blit( otext, (x-owidth, y) )
                        screen.blit( otext, (x+owidth, y) )
                        screen.blit( otext, (x, y-owidth) )
                        screen.blit( otext, (x, y+owidth) )

                if outline == "outrect":
                        dim( screen, oalpha, ocolor, (x-owidth, y-owidth, rtx+owidth*2, rty+owidth) )

                if outline == "shadow":
                        otext = font.render( text, True, ocolor )
                        otext.set_alpha( oalpha )				# Worked once, but now it doesn't... don't know why
                        screen.blit( otext, (x+owidth, y+owidth) )

                if outline == "FadeIn":
                        rect = ( x, y, rtx, rty )
                        srf = pygame.Surface( (rect[2],rect[3]) )
                        srf.blit( rtext, (0,0) )
                        img = Screenshot( srf, (0,0), (rect[2],rect[3]) )
                        FadeIn( screen, img, rect )

                if outline == "FadeOut":
                        rect = ( x, y, rtx, rty )
                        srf = pygame.Surface( (rect[2], rect[3]) )
                        srf.blit( rtext, (0,0) )
                        img = Screenshot( srf, (0,0), (rect[2],rect[3]) )
                        FadeOut( screen, img, rect )

                if outline != "FadeOut": screen.blit( rtext, (x, y) )

	rtx = rtx + owidth*2
	rty = rty + owidth

	return (rtx,rty)

####################################################################
def Screenshot( screen, pos, size ):
        
        img = pygame.Surface( size )
        img.blit( screen, (0,0), (pos,size) )
    
        return img

####################################################################
def FadeIn( screen, img, rect, time=1, color_filter=(0,0,0) ):

        clock = pygame.time.Clock()
        darken_factor = 255
        darken_step = 25.5
        fadeFps = int( ( 255 / darken_step ) / time )
        
        while darken_factor > 0:
                clock.tick( fadeFps )
                screen.blit( img, (rect[0],rect[1]) )
                dim( screen, darken_factor, color_filter, rect, True )
                darken_factor = int( darken_factor - ( darken_step / time ) )
                event_loop()

        screen.blit( img, (rect[0],rect[1]) )
        pygame.display.update( rect )
        
        return

####################################################################
def FadeOut( screen, img, rect, time=1, color_filter=(0,0,0) ):

        clock = pygame.time.Clock()
        darken_factor = 0
        darken_step = 25.5
        fadeFps = int( ( 255 / darken_step ) / time )
        
        while darken_factor < 255:
                clock.tick( fadeFps )
                screen.blit( img, (rect[0],rect[1]) )
                dim( screen, darken_factor, color_filter, rect, True )
                darken_factor = int( darken_factor + ( darken_step / time ) )
                event_loop()

        pygame.draw.rect( screen, color_filter, rect )
        pygame.display.update( rect )

        return

"""
        # FadeIn/FadeOut EXAMPLE
        # MUST use a different Surface to blit the text/image first, then fade it on main Surface
        srf = pygame.Surface( (rect[2], rect[3]) )
        pygame.draw.rect( srf, nBkg, (0,0,rect[2],rect[3]) )
        font = pygame.font.SysFont( self.fn, size, bold)
        rtext = font.render( "Texto de prueba para FadeIn / FadeOut", True, white )
        srf.blit( rtext, (0,0) )
        img = Screenshot( srf, (0,0), (rect[2],rect[3]))
        while True:
            FadeIn( self.screen, img, rect, 2, nBkg )
            sleep(5)
            FadeOut( self.screen, img, rect, 2, nBkg )
"""
