#!/usr/bin/python
# -*- coding: utf-8 -*-

import locale

# GENERAL Parameters
dispSize = (1024,600)				# Set the resolution (x, y). 'None' for display's native resolution (fullscreen)
optimdispSize = (1024,600)                      # Screen size your script is intended for (bkg and icons sizes, mainly)
disp_units = 'metric'				# Set 'metric' or 'imperial'
lang = 'es'					# Set 'es' (Spanish) or 'en' (English)
if lang == 'es': locale.setlocale(locale.LC_ALL, 'es_ES.UTF-8')		# Use language locale or install it (es_ES/en_GB)
else: locale.setlocale(locale.LC_ALL, 'en_GB.UTF-8')  			# using 'sudo dpkg-reconfigure' locales and select
font = 'freesans'				# General font. If not available, all available fonts will be printed. Choose one!
calfont = 'freemono'

#APPEARANCE Parameters (Note there are other parameters you may change within "Display" functions)
#MORE Parameters in __init__ function (screen size-dependent)
bkgf = 'wbkg/'					# Background images folder location
iconf = 'icons/'				# Weather Icons folder location
iconfB = 'iconsB/'
showBkg = True					# Show picture background (or solid color if False)
if showBkg: ol = 'outline'			# Outline text for better visualization (or choose 'outrect' or 'shadow' options)
else: ol = 'shadow'				# Not needed if not bkg shown
dimBkg = 'all'                                  # When to dim bkg's: "all" (always), "cc" (current conds area), "light" (only light bkg's)
dimFactor = 64					# Darken factor when dimming Bkg (or part of it)
lightBkg = (16,27,28,42,43)			# Determine light backgrounds for better visualization when dimBkg = "light"
nPeriod = (0,15,30,45)				# Periods smaller than the duration of the News (nTime), will be ignored
wPeriod = (6,10,14,21,25,29,36,40,44,51,55,59)	# Weather update periods. MUST be "outside" News periods (calculated as nPeriod+nTime)
NSub = 4					# Number of Forecast Subwindows
lines = 1					# Number of lines to be drawn at a time (lines thickness)
showNightf = False                              # Show Night forecast icon (not suitable for small and medium displays)

#COLORS
black = (0,0,0)
white = (255,255,255)
lightGrey = (200,200,200)
dmedGrey = (100,100,100)
lmedGrey =(150,150,150)
vlmedGrey = (175,175,175)
darkGrey = (64,64,64)
orange = (255,128,0)
lightOrange = (255,153,51)
red = (204,0,0)
blue = (0,0,204)
medBlue = (0,76,153)
lightBlue = (0,120,255)
vlightBlue = (153, 204, 255)

tdBkg = medBlue				        # Time and Date background color (None if transparent)
cBkg = black					# "Middle" (current conditions) background color (will not apply if showBkg=True)
nBkg = medBlue					# News background color

if showBkg:					# Highlight text when background active
	lc = white				# General Text color
	llc = lmedGrey				# Lines color
	cccW = lc	 			# Outside temp - Warm
	cccC = lc				# Outside Temp - Cold
	cccTCW = lc				# Outside Temp - Too Cold or Too Warm
	ctc = vlmedGrey				# Conditions - Titles
else:
	lc = lightGrey				# General Text color
	llc = dmedGrey				# Lines color
	cccW = lightOrange 			# Outside temp - Warm
	cccC = lightBlue			# Outside Temp - Cold
	cccTCW = red				# Outside Temp - Too Cold or Too Warm
	ctc = lmedGrey				# Conditions - Titles
dtc = lc					# Header Text color (Be aware to control Day/Night colors in time_date)
dtcNight = lightGrey				# Header Text color at Night
ccc = lc					# Outside Temp - Neutral
cccNight = lightGrey				# Outside Temp at Night
cvc = lc					# Conditions - Values
cws = lc					# Windspeed Alert - Regular
cwsH = red					# Windspeed Alert - High
crc = lc					# Rain precipitation probability - Null
crcm = lightOrange				# Rain precipitation probability - Low-Medium
crcw = red					# Rain precipitation probability - High
loc = lightOrange				# Location and "by" color (not affected by background)
fHTc = red					# Forecasts High Temp outline color
fLTc = blue					# Forecasts Low Temp outline color
nc = lc

#TEXT SIZE - WARNING: They all are relative to screen size (Yaxis)!
#Large Display
rTh = 0.035					# Regular Font
dTh = 0.0675					# Date Text Height
tmTh = dTh*2.4					# Time Text Height (Hours:Minutes)
tmSTh = tmTh/2					# Seconds Text Height
lSmTh = dTh/2					# Location and update Text Height
sTh = lSmTh*1.5					# Source text Height
oTh = 0.27					# Outside temperature Text Height
oDTh = 0.12					# Degrees symbol for outside temperature Text Height
oTTh = 0.045					# Outside conditions text
swTh = 0.05					# Sub window text height
swRTh = swTh*2.2				# Rain chance text height
#Small Display - NOT WELL TESTED!!!
rThS = 0.03
dThS = 0.064					# Date Text Height
tmThS = dThS*2.4				# Time Text Height (Hours:Minutes)
tmSThS = tmThS/2				# Seconds Text Height
lSmThS = dThS/2.5				# Location and update Text Height
sThS = lSmThS*1.5				# Source text Height
oThS = 0.25					# Outside temperature Text Height
oDThS = 0.1					# Degrees symbol for outside temperature Text Height
oTThS = 0.04					# Outside conditions text
swThS = 0.04					# Sub window text height
swRThS = swThS*2				# Rain chance text height

#POSITIONS - WARNING: they all are relative to screen size!
#Time, Date, Location Positions
txp = 0.5					# Time Xaxis position
tmdateYPos = 0.0				# Time & Date Yaxis Position
tmdateYPosSm = 0.017				# Time & Date Yaxis Position Small (seconds)
lp = 0.75					# Location Xaxis position
#Current Conditions Positions
tp = 0.25					# Outside temp Yaxis position
dp = tp + 0.02					# Outside temp degrees sign Yaxis position
ixp = 0.25					# Current cond icon Xaxis position
iyp = 0.15					# Current cond icon Yaxis position
#Other Conditions Positions
st = 0.285					# Yaxis Start Pos
gp2 = 0.06					# Line Spacing Gap
xp = 0.75      					# Xaxis Start Pos
x2 = 0.87					# Second Column Xaxis Start Pos
#Sub-Windows Positions
wx = 1.0/(NSub*2)				# Sub Window Centers
wy = 0.65					# Sub Windows Yaxis Start
gp = 0.045					# Line Spacing Gap

# WEATHER Parameters
# Note that other providers may (will) have slight differences in their APIs
wsource = 'Weather.com'				# Use 'Weather.com' or 'Yahoo!' (Yahoo! has no rain chance)
if ( wsource == 'Yahoo!' ):
        weatherURL = 'https://query.yahooapis.com/v1/public/yql?q=select * from weather.forecast where woeid = "%s" and u="%s"&format=json&env=store://datatables.org/alltableswithkeys'
        zip_code = ['766273',
                    '762109',
                    '472553'
                    ]                 # Your Yahoo! woeid code here. Search for it at < http://woeid.rosselliot.co.nz/ >
	if disp_units == 'metric': query_units = 'c'
	else: query_units = 'f'
	qu = 'query'
	re = 'results'
	ch = 'channel'
	it = 'item'
	cc = 'condition'
	f = 'forecast'
else:
        weatherURL = 'http://wxdata.weather.com/wxdata/weather/local/%s?unit=%s&dayf=4&cc=*'  # Will return XML, not JSON
        zip_code = ['SPMD0594',
                    'SPXX0003',
                    'ANXX0001'
                    ]                 # YOUR ZIP code here. Search for it at < http://weather.codes/ >
	if disp_units == 'metric': query_units = 'm'
	else: query_units = "i"
	cc = 2
	f = 3
degree_sign= u'\N{DEGREE SIGN}'			# Degree symbol
if disp_units == 'metric':
	uniTmp = unichr(0x2103)			# Unicode for DegreeC
	windSpeed = 'Km/h'
	windScale = 1.0
	baroUnits = ' mb'
        baroScale = 29.529980164712
else:
	uniTmp = unichr(0x2109)			# Unicode for DegreeF
	windSpeed = 'mph'
	windScale = 1.0
	baroUnits = ' "Hg'
	baroScale = 1.0
WarmTemp = 30					# Temperature Alert upper threshold (Hot)
TooWarmTemp = 38
ColdTemp = 18					# Temperature Alert lower threshold (Cold)
TooColdTemp = 4
WindAlert = 60					# Wind Alert threshold
RainHigh = 30					# Rain chance upper threshold
if disp_units == 'imperial':
        WarmTemp = WarmTemp * 1.8 + 32
	TooWarmTemp = TooWarmTemp * 1.8 + 32
	ColdTemp = ColdTemp * 1.8 + 32
	TooColdTemp = TooColdTemp * 1.8 + 32
	WindAlert = WindAlert / 1.609344

# NEWS Parameters
nsource1 = 'rtve'
nURL1 = "http://www.rtve.es/api/noticias.json?size=5"
nsource2 = 'BBC'
nURL2 = "http://feeds.bbci.co.uk/news/rss.xml"
alternSource = True				# Changes from one source to another in every update
nTime = 5*60					# Time in seconds for showing News (or altern between sources and update Weather when in 'alwyaysON' mode)
NewsMode = 'period'				# News Mode: 'period', 'alwaysON', 'alwaysOFF'
nPos = 0.59					# News ticker position on screen (0.65 for 1920x1080)
nPosB = 0.65
dimBkgNews = False				# Dim screen when showing News (it will add to general dimBkg parameter)
nSize = 60					# Size font for News
nBold = True					# Bold font for News
fps = 25					# Reducing fps will consume less CPU, but the ticker will slow down

# CLOCK Parameters
ifname = 'enp3s0'                               # Your network interface here (check ifconfig on a terminal, it should look like 'eth0' or 'wlan0')
clockBkg = black                                # Background color when in Clock mode
clockc = vlightBlue                             # General text color for Clock
clockh = dmedGrey                               # Calendar header text color
clockYPos = 0.25
calYPos = 0.03
