#!/usr/bin/python
# -*- coding: utf-8 -*-

import locale

# GENERAL Parameters
dispSize = (1024,600)                           # Set the resolution (x, y). 'None' for display's native resolution (fullscreen)
optimdispSize = (1024,600)                      # Screen size your script is intended for (will affect bkg and icons sizes, mainly)
disp_units = 'metric'                           # Set 'metric' or 'imperial'
lang = 'es'                                     # Set 'es' (Spanish) or 'en' (English)
if lang == 'es': locale.setlocale(locale.LC_ALL, 'es_ES.UTF-8') # Use language locale or install it (es_ES/en_GB)
else: locale.setlocale(locale.LC_ALL, 'en_GB.UTF-8')            # using 'sudo dpkg-reconfigure' locales and select
font = 'freesans'                               # General font. If not available, all available fonts will be printed. Choose one!
calfont = 'dejavusansmono'                      # "mono" (non-proportional font for calendar
timefont = 'robotocondensed'                    # Time (clock) font

#APPEARANCE Parameters (Note there are other parameters you may change within "Display" functions)
#MORE Parameters in __init__ function (screen size-dependent)
bkgf = 'wbkg/'                                  # Background images folder location
showBkg = False                                 # Show background
Bkg = ''                                        # Background file name (must be on the same folder than main script)
iconSet = 'D'                                   # 'A' - Full color / 'B' - Flat Grey / 'C' - Sticky full color / 'D' - Flat full color
if iconSet in ('A','C'):                        # These icon sets need additonal scalation
        iconScale = 0.9
else: iconScale = 1.0
iconf = 'icons'+iconSet+'/'                     # Icons folder location
iconW = iconf+'30.png'                          # System icon for the display window (must be located in the icons folder)
caption = 'Weather & News by alef'              # Window title
wPeriod = (6,10,14,21,25,29,36,40,44,51,55,59)  # Weather update periods. MUST be "outside" News periods (calculated as nPeriod+nTime)
NewsMode = 'period'                             # News Mode: 'period', 'alwaysON', 'alwaysOFF'
nPeriod = (0,15,30,45)                          # Periods smaller than the duration of the News (nTime), will be ignored
nTime = 5*60                                    # Time in seconds for showing News (or altern between sources and update Weather when in 'alwyaysON' mode)
alternSource = True                             # Changes from one source to another in every update
NSub = 4                                        # Number of Forecast Subwindows
lines = 1                                       # Number of lines to be drawn at a time (lines thickness)

#COLORS
black = (0,0,0)
white = (255,255,255)
lightGrey = (200,200,200)
dmedGrey = (100,100,100)
lmedGrey =(150,150,150)
vlmedGrey = (175,175,175)
darkGrey = (64,64,64)
vdarkGrey = (32,32,32)
orange = (255,128,0)
lightOrange = (255,153,51)
red = (204,0,0)
turquoise = (0,204,204)
blue = (0,0,204)
medBlue = (0,76,153)
lightBlue = (51,153,255)
vlightBlue = (102, 178, 255)
ulightBlue = (153, 204, 255)

cBkg = black                                    # General background color
nBkg = black                                    # News background color
wBkg = black                                    # Weather backgroun color (Location not included)

clockc = turquoise                              # General text color for Clock
clockh = dmedGrey                               # Calendar header text color
nc = orange                                     # News color
wc = turquoise                                  # Weather text color

crcm = orange                                   # Rain precipitation probability - Low-Medium
crcw = red                                      # Rain precipitation probability - High
byc = orange                                    # Location and "by" color (not affected by background)
llc = clockc                                    # Lines color

#TEXT SIZE - WARNING: They all are relative to screen size (Yaxis)!
if int(dispSize[0]) > 800: Th = 0.0675          # Large display Text Size
else: Th = 0.05                                 # Small Display Text Size
byTh = Th*0.75
wdayTh = Th*0.8
mdayTh = Th*1.25
yearTh = wdayTh
calTh = Th/2
timeTh = Th*7
secTh = timeTh/2
cityTh = Th*1.2
statTh = cityTh*0.6
tempTh = Th*3.3
degTh = tempTh/2.25
temptxTh = Th*0.7
condTh = Th/2.7
subtempTh = Th*0.8
subrainTh = Th*1.1
nSize = 60                                      # News font size
nBold = True                                    # Bold font for News
if int(dispSize[0]) <= 800: nSize = 40          # Small Display News font size

#POSITIONS - WARNING: they all are relative to screen size!
#Time, Date, Location Positions
clockYPos = 0.02                                # Clock Yaxis position
calYPos = 0.03                                  # Calendar Yaxis position
subXPos = 0.53                                  # Sub Windows (forecasts) Xaxis Start
subYPos = 0.68                                  # Sub Windows (forecasts) Yaxis Start
if int(dispSize[0]) > 800: nPos = 0.57          # News ticker Yaxis position
else: nPos = 0.5

# WEATHER Parameters
# Note that other providers may (will) have slight differences in their APIs
wsource = 'Weather.com'                         # Use 'Weather.com' or 'Yahoo!' (Yahoo! has no rain chance)
if ( wsource == 'Yahoo!' ):
        weatherURL = 'https://query.yahooapis.com/v1/public/yql?q=select * from weather.forecast where woeid = "%s" and u="%s"&format=json&env=store://datatables.org/alltableswithkeys'
        zip_code = ['766273',
                    '762109',
                    '472553'
                    ]                 # Your Yahoo! woeid code here. Search for it at < http://woeid.rosselliot.co.nz/ >
        if disp_units == 'metric': query_units = 'c'
        else: query_units = 'f'
        qu = 'query'
        re = 'results'
        ch = 'channel'
        it = 'item'
        cc = 'condition'
        f = 'forecast'
else:
        weatherURL = 'http://wxdata.weather.com/wxdata/weather/local/%s?unit=%s&dayf=4&cc=*'  # Will return XML, not JSON
        zip_code = ['SPMD0594',       # SPXX0050
                    'SPXX0003',
                    'ANXX0001'
                    ]                 # YOUR ZIP code here. Search for it at < http://weather.codes/ >
        if disp_units == 'metric': query_units = 'm'
        else: query_units = "i"
        cc = 2
        f = 3
degree_sign= u'\N{DEGREE SIGN}'                 # Degree symbol
if disp_units == 'metric':
        uniTmp = unichr(0x2103)                 # Unicode for DegreeC
        windSpeed = 'Km/h'
        windScale = 1.0
        baroUnits = ' mb'
        baroScale = 29.529980164712
else:
        uniTmp = unichr(0x2109)                 # Unicode for DegreeF
        windSpeed = 'mph'
        windScale = 1.0
        baroUnits = ' "Hg'
        baroScale = 1.0
RainHigh = 40                                   # Rain chance upper threshold

# NEWS Parameters
nsource1 = 'rtve'
nURL1 = "http://www.rtve.es/api/noticias.json?size=5"
nsource2 = 'BBC'
nURL2 = "http://feeds.bbci.co.uk/news/rss.xml"
separator = "   |   "                           # Separation characters between News
fps = 25                                        # Reducing fps will consume less CPU, but the ticker will slow down
