#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017 Angel Luis Escribano <alescribano@gmail.com>
################################################################
# WHAT IS THIS FOR?
#   - Shows Weather (current and forecasts) from Weather.com
#   - Latest News from RTVE or BBC 
# WARNING: Should you want other sources, you will have to adapt/struggle with the APIs
################################################################
# Based on the work of many people. Thank you for sharing!!!
#    WEATHER: Jim Kemp < https://www.instructableS.com/id/Raspberry-Pi-Internet-Weather-Station/ >
#    ICONS: VClouds http://vcloudS.deviantart.com/ 
#    BACKGROUNDS: Dr. Toxic and Timdog82001
#    NEWS TICKER (ScrollText): Gunny26 < https://github.com/gunny26/pygame/blob/master/ScrollText.py >
#    SCREEN dimMER: Thobias Thelen < http://www.pygame.org/pcr/screen_dimmer/index.php >
#    TFT GPIO SCREENS: wborelli < https://www.instructableS.com/id/Raspberry-Pi-Internet-Weather-Station/ > (NOT Tested)
################################################################
### BEGIN LICENSE
# See README.TXT File
### END LICENSE
################################################################

__version__ = "1.6.1"

import pygame
import time
import string
import urllib
import contextlib
import json
import xml.etree.ElementTree as ET
import calendar
import settingsA as S
import disputil
import wtranslat
import scrolltx

###############################################################################
def getIcon( code ):
    try:
        # Add .convert_alpha() if you find transparency troubles showing icon
        icon = pygame.image.load( S.iconf + str(code) + ".png" )
    except:
        icon = pygame.image.load( S.iconf + "na.png" )
    return icon

###############################################################################
def getBkg( code ):
    try:
        # Add .convert_alpha() if you find transparency troubles showing bkg
        bkg = pygame.image.load( S.bkgf + str(code) + ".jpg" )
    except:
        bkg = pygame.image.load( S.bkgf + "na.jpg" )
    return bkg

###############################################################################

class MyDisplay:
    screen = None;
    
    ####################################################################
    def __init__(self):

        if S.dispSize != None:
            # Change S.dispSize to try other resolutions (tested: 1920x1080, 1024x600, 800x600)
            # If S.dispSize matches screen size, S.it will turn into fullscreen mode and mouse will be hided
            self.screen, self.xmax, self.ymax = disputil.InitDisplay( S.dispSize[0], S.dispSize[1], windowed=True )
        else:
            self.screen, self.xmax, self.ymax = disputil.InitDisplay()
        self.xmin = self.ymin = 0
        self.xmargin = self.xmax*0.01
        self.ymargin = self.ymax*0.01

        self.fn = disputil.InitFonts( S.font )

        self.pbkgCode = None            # Initial previous background (load new bkg only when changes)
        # These values will change according to some conditionS. "Saving" them to self. variables
        self.dtc = S.dtc
        self.ccc = S.ccc
        self.cws = S.cws
        self.crc = S.crc
        self.nsource = S.nsource1
        self.nURL = S.nURL1
        self.zip_code = S.zip_code[0]
        self.keyP = None
        self.modeNight = False
        self.onlyTime = True
        self.errCount = 0
        self.nPos = S.nPos

        self.location = ''
        self.iconNow = 'na'
        self.bkgCode = 'na'
        self.temp = ''
        self.temptext = ''
        self.feels_like = '0'
        self.wind_speed = '0'
        self.baro = '29.95'
        self.wind_dir = 'S'
        self.humid = '50.0'
        self.wLastUpdate = ''
        self.last = ''
        self.day = [ '', '', '', '' ]
        self.icon = [ 0, 0, 0, 0 ]
        self.iconNight = [ 0, 0, 0, 0 ]
        self.rain = [ '', '', '', '' ]
        self.temps = [ ['',''], ['',''], ['',''], ['',''] ]
        self.sunrise = '7:00 AM'
        self.sunset = '8:00 PM'

        # Icon scaling according to resolution and size (mine are optimized for 1024x600)
        if ( (self.xmax, self.ymax) == S.optimdispSize ):
            self.scaleIcon = False
        else:
            self.scaleIcon = True
        # Try this formula to calculate desired size (128 is the target for 1024x600, but will not always work!!!)
        #self.iconScaleSub =  ( ( 128.0 * self.xmax ) / 1024.0 ) /  iconSize
        self.iconScaleSub = self.xmax / float(S.optimdispSize[0])
        self.iconScale = self.iconScaleSub * 2.6        # Icon scale amount for main icon (current conditions)

        if self.xmax > 800:
            # Large Display
            self.regTh = int(self.ymax*S.rTh)             # Regular Font
            self.dateTh = int(self.ymax*S.dTh)            # Date Text Height
            self.tmTh = int(self.ymax*S.tmTh)             # Time Text Height (Hours:Minutes)
            self.tmSTh = int(self.ymax*S.tmSTh)           # Seconds Text Height
            self.tmTh2 = int(self.ymax*S.tmTh)*2          # Time Text Height (Hours:Minutes)
            self.tmSTh2 = self.tmTh2/3                    # Seconds Text Height
            self.byTh = int(self.ymax*S.sTh)              # "by" text Height
            self.lSmTh = int(self.ymax*S.lSmTh)           # Location and update Text Height
            self.sourceTh = int(self.ymax*S.sTh)          # Source text Height
            self.outTempTh = int(self.ymax*S.oTh)         # Outside temperature Text Height
            self.outTempDTh = int(self.ymax*S.oDTh)       # Degrees symbol for outside temperature Text Height
            self.outTempTTh = int(self.ymax*S.oTTh)       # Outside conditions text height
            self.subwinTh = int(self.ymax*S.swTh)         # Sub window text height
            self.subwinRTh = int(self.ymax*S.swRTh)       # Rain chance text height
        else: 
            # Small Display - NOT FULLY TESTED!!!!
            self.regTh = int(self.ymax*S.rThS)           # Regular Font
            self.dateTh = int(self.ymax*S.dThS)          # Date Text Height
            self.tmTh = int(self.ymax*S.tmThS)           # Time Text Height (Hours:Minutes)
            self.tmSTh = int(self.ymax*S.tmSThS)         # Seconds Text Height
            self.tmTh2 = int(self.ymax*S.tmThS)*2         # Time Text Height (Hours:Minutes)
            self.tmSTh2 = self.tmTh2/3                    # Seconds Text Height
            self.byTh = int(self.ymax*S.sThS)             # "by" text Height
            self.lSmTh = int(self.ymax*S.lSmThS)         # Location and update Text Height
            self.sourceTh = int(self.ymax*S.sThS)        # Source text Height
            self.outTempTh = int(self.ymax*S.oThS)       # Outside temperature Text Height
            self.outTempDTh = int(self.ymax*S.oDThS)     # Degrees symbol for outside temperature Text Height
            self.outTempTTh = int(self.ymax*S.oTThS)     # Outside conditions text height
            self.subwinTh = int(self.ymax*S.swThS)       # Sub window text height
            self.subwinRTh = int(self.ymax*S.swRThS)     # Rain chance text height
        self.tmdateYPos2 = self.ymax*S.clockYPos          # Clock position
        self.calYPos = self.ymax*S.calYPos                # Calendar position

    ####################################################################
    def __del__(self):
        "Destructor to make sure pygame shuts down, etc."

    ####################################################################
    def ParseWeathercom( self, w ):

        wUpdated = False

        if ( w[S.cc][0].text != self.wLastUpdate ):
            wUpdated = True
            self.last = time.strftime( '%H:%M')

            # Current Conditions
            self.wLastUpdate = w[S.cc][0].text
            self.location = w[S.cc][1].text
            self.temp = w[S.cc][2].text
            self.feels_like = w[S.cc][3].text + S.degree_sign
            self.iconNow = w[S.cc][5].text
            if self.iconNow == None: self.iconNow = w[S.f][1][4][0].text
            if self.iconNow == None: self.iconNow = w[S.f][1][5][0].text
            self.bkgCode = self.iconNow
            if S.lang  == 'en': self.temptext = string.capitalize( w[S.cc][4].text )
            else: self.temptext = string.capitalize( wtranslat.TranslateT( S.lang, int( self.iconNow ) ) )
            self.baro = w[S.cc][6][0].text
            self.wind_speed = string.upper ( w[S.cc][7][0].text )
            if 'CALM' in self.wind_speed: self.wind_speed = 0
            else: self.wind_speed = int(self.wind_speed) * S.windScale
            self.wind_dir = string.upper( w[S.cc][7][3].text )
            if 'VAR' in self.wind_dir: self.wind_dir = 'Var'
            if 'CALM' in self.wind_dir: self.wind_dir = '--'
            if S.lang == 'es': self.wind_dir = self.wind_dir.replace( "W", "O" )
            self.humid = w[S.cc][8].text
            self.sunrise = w[1][4].text
            self.sunset = w[1][5].text
            if ( self.sunrise[1] == ":" ): self.sunrise = "0" + self.sunrise
            self.sr = int( self.sunrise[:2] )
            if ( self.sunset[1] == ":" ): self.sunset = "0" + self.sunset
            self.sn = int( self.sunset[:2] ) + 12

            # Forecasts
            i = 0
            for day in w.findall('./dayf/day'):
                self.day[i] = wtranslat.TranslateD( S.lang, w[S.f][i+1].get('t') )
                self.icon[i] = w[S.f][i+1][4][0].text
                self.iconNight[i] = w[S.f][i+1][5][0].text
                self.rain[i] = w[S.f][i+1][4][4].text
                self.temps[i][0] = w[S.f][i+1][0].text + S.degree_sign
                self.temps[i][1] = w[S.f][i+1][1].text + S.degree_sign
                if i == 0:
                    self.day[i] = wtranslat.TranslateD( S.lang, "Today" )
                    # At some moment, some info for Today is null, taking "Night part"
                    if self.icon[i] == None:
                        self.icon[i] = w[S.f][i+1][5][0].text
                        self.iconNight[i] = None
                        self.rain[i] = w[S.f][i+1][5][4].text
                i += 1

            hh = int(self.last[:2])
            if ( ( ( hh > 12 ) and ( hh > int(self.sn) ) ) or ( ( hh < 12 ) and ( hh < self.sr ) ) ):
                self.modeNight = True
            else: self.modeNight = False

        return wUpdated

    ####################################################################
    def ParseYahooWeather( self, w ):

        wUpdated = False

        if ( w[S.qu][S.re][S.ch][S.it]['pubDate'] != self.wLastUpdate ):
            wUpdated = True
            self.last = time.strftime( '%H:%M')

            # Current Conditions
            self.wLastUpdate = w[S.qu][S.re][S.ch][S.it]['pubDate']
            self.location = w[S.qu][S.re][S.ch]['title']
            self.location = self.location[17:]
            self.iconNow = w[S.qu][S.re][S.ch][S.it][S.cc]['code']
            self.bkgCode = self.iconNow
            self.temp = w[S.qu][S.re][S.ch][S.it][S.cc]['temp']
            if S.lang  == 'es': self.temptext = string.capitalize( wtranslat.TranslateT( S.lang, int( self.iconNow ) ) )
            else: self.temptext = string.capitalize( w[S.qu][S.re][S.ch][S.it][S.cc]['text'] )
            self.feels_like = w[S.qu][S.re][S.ch]['wind']['chill'] + S.degree_sign
            if S.disp_units == 'metric': self.feels_like = str( int ( ( int( self.feels_like[:-1] ) - 32 ) / 1.8 ) ) + S.degree_sign
            self.wind_speed = float( w[S.qu][S.re][S.ch]['wind']['speed'] ) * S.windScale
            self.wind_dir = wtranslat.TranslateWindDir( float( w[S.qu][S.re][S.ch]['wind']['direction'] ) )
            if S.lang == 'es': self.wind_dir = self.wind_dir.replace( "W", "O" )
            self.baro = w[S.qu][S.re][S.ch]['atmosphere']['pressure']
            if S.disp_units == "metric": self.baro = str( round( float( self.baro ) / S.baroScale, 2 ) )
            self.humid = w[S.qu][S.re][S.ch]['atmosphere']['humidity']
            self.sunrise = w[S.qu][S.re][S.ch]['astronomy']['sunrise'] 
            self.sunset = w[S.qu][S.re][S.ch]['astronomy']['sunset']
            if ( self.sunrise[1] == ":" ): self.sunrise = "0" + self.sunrise
            self.sr = int( self.sunrise[:2] )
            if ( self.sunset[1] == ":" ): self.sunset = "0" + self.sunset
            self.sn = int( self.sunset[:2] ) + 12

            # Forecasts
            for i in range (0,4):
                if i == 0: self.day[i] = wtranslat.TranslateD( S.lang, "Today" )
                else: self.day[i] = w[S.qu][S.re][S.ch][S.it][S.f][i]['day']
                self.icon[i] = w[S.qu][S.re][S.ch][S.it][S.f][i]['code']
                self.iconNight[i] = None
                self.rain[i] = str( 0 )  # No precipitation probability in API :(
                self.temps[i][0] = w[S.qu][S.re][S.ch][S.it][S.f][i]['high'] + S.degree_sign
                self.temps[i][1] = w[S.qu][S.re][S.ch][S.it][S.f][i]['low'] + S.degree_sign

            hh = int(self.last[:2])
            if ( ( ( hh > 12 ) and ( hh > int(self.sn) ) ) or ( ( hh < 12 ) and ( hh < self.sr ) ) ):
                self.modeNight = True
            else: self.modeNight = False

        return wUpdated

    ####################################################################
    def UpdateWeather( self, firstRun=False ):

        wUpdated = False
        
        # Get Weather information from source
        url = S.weatherURL % ( self.zip_code, S.query_units )
        try:
            with contextlib.closing(urllib.urlopen(url)) as response:
                if S.wsource == "Yahoo!":
                    w = json.loads( response.read() )
                    wUpdated = self.ParseYahooWeather( w )
                else:
                    w = ET.fromstring( response.read() )
                    wUpdated = self.ParseWeathercom( w )
            if self.onlyTime:
                self.onlyTime = False
                self.nPos = S.nPos
                wUpdated = True
            self.errCount = 0

        except KeyError:
            print "KeyError -> Weather Error"
        except ValueError:
            print "ValueError -> Weather Error"
        except:
            print "Error getting update from " + S.wsource + " " + self.last
            self.errCount += 1
            if firstRun or self.errCount > 10:
                self.onlyTime = True
                self.nPos = S.nPos+0.1

        return wUpdated

    ####################################################################
    def DisplayTime( self, blit=True ):

        if self.onlyTime:
            myDisp.DisplayOnlyTime()
            return

        tm2 = time.strftime( "%H:%M", time.localtime() )                # Hour:Minutes
        tm3 = time.strftime( "%S", time.localtime() )                   # Seconds

        (tx2, ty2) = disputil.DrawText( self.screen, tm2, self.fn, self.tmTh, 1, S.dtc, blit=False )
        (tx3, ty3) = disputil.DrawText( self.screen, tm3, self.fn, self.tmSTh, 1, S.dtc, blit=False )

        owidth = 3
        rect = (self.xmax/2-tx2/2-owidth*2, self.ymin, tx2+tx3+owidth*4, ty2-self.ymargin-owidth)

        if blit:
            if S.tdBkg == None and S.showBkg:
                self.screen.blit( self.bkgPart, rect )
                outline = 'outline'
            else:
                pygame.draw.rect( self.screen, S.tdBkg, rect )
                outline = None

            disputil.DrawText( self.screen, tm2, self.fn, self.tmTh, 1, self.dtc, blit=True, x=self.xmax/2-tx2/2, y=self.ymax*S.tmdateYPos, outline=outline, owidth=owidth )
            disputil.DrawText( self.screen, tm3, self.fn, self.tmSTh, 1, self.dtc, blit=True, x=self.xmax/2+tx2/2, y=S.tmdateYPos+(self.ymax*S.tmdateYPosSm), outline=outline, owidth=owidth )

            pygame.display.update( rect )

        return rect

    ####################################################################
    def DisplayOnlyTime( self, firstRun=False ):

        xgap = self.xmargin*3
        ygap = self.ymargin*3
        columnWidth = 3

        tm2 = time.strftime( "%H:%M", time.localtime() )                                # Hour:Minutes
        tm3 = time.strftime( "%S", time.localtime() )                                   # Seconds
        (tx2,ty2) = disputil.DrawText( self.screen, tm2, self.fn, self.tmTh2, 0, S.clockc, blit=False )
        (tx3,ty3) = disputil.DrawText( self.screen, tm3, self.fn, self.tmSTh2, 1, S.clockc, blit=False )
 
        if ( int(tm2[:2])+int(tm2[-2:])+int(tm3) == 0 ) or firstRun:
            print "All"

            self.screen.fill( S.clockBkg )

            tm0 = "by alef"
            tm1a = wtranslat.elimina_tilde( time.strftime( "%A", time.localtime() ) )   # Day of week
            tm1b = time.strftime( "%B %d", time.localtime() )                           # Date
            tm4 = time.strftime( "%Y", time.localtime() )                               # Year
            century = wtranslat.TranslateT( S.lang, 107 )
            tm5 = "  %s XXI" % century                                                  # Century
            c = calendar.TextCalendar()                                                 # Calendar
            year = int( tm4 )
            month = int( time.strftime("%m", time.localtime()) )
            day = int(time.strftime( "%d", time.localtime()) )
            cal = c.formatmonth( year, month, w=columnWidth ).splitlines()              # Calendar splitted into separate lines

            (tx0,ty0) = disputil.DrawText( self.screen, tm0, self.fn, self.byTh, 0, S.loc, blit=False )
            (tx1a,ty1a) = disputil.DrawText( self.screen, tm1a, self.fn, self.dateTh/2, 0, S.clockc, blit=False )
            (tx1b,ty1b) = disputil.DrawText( self.screen, tm1b, self.fn, self.dateTh, 0, S.clockc, blit=False )
            (tx4,ty4) = disputil.DrawText( self.screen, tm4, self.fn, int(self.dateTh*1.5), 1, S.clockc, blit=False )
            (tx5,ty5) = disputil.DrawText( self.screen, tm5, self.fn, self.dateTh, 0, S.clockc, blit=False )

            # Calendar
            self.fn = disputil.InitFonts( S.calfont )                                   # Using a "mono" non-proportional font
        
            if S.lang == 'es': cal[1] = wtranslat.TranslateT( S.lang, 108 )             # Setting spanish header (troubles with accents)
            (tx,ty) = disputil.DrawText( self.screen, cal[1], self.fn, self.dateTh/2, 0, S.clockc, blit=False )
            x = ix = self.xmax - tx - xgap
            y = iy = self.calYPos + ty1a + ty1b + ygap
        
            # Draw calendar
            for i in range (1,len(cal)):
                if i == 1: color = S.clockh
                else: color = S.clockc
                disputil.DrawText( self.screen, cal[i], self.fn, self.dateTh/2, 0, color, blit=True, x=x, y=y)
                y = y + ty + ygap
            
            # Highlight current day
            (leapDay,daysTotal) = calendar.monthrange( year, month )
            # Find column
            column = day % 7 + leapDay
            if column > 7: column = column - 7
            # Find row
            row = ((day + leapDay) / 7)
            if (day + leapDay) % 7 != 0: row = row + 1
            # Highlight
            rx = ix + ( column * tx / 7 ) - xgap - columnWidth + column
            ry = iy + ( row * ( ty + ygap ) ) - columnWidth
            w = h = ty + columnWidth * 3
            rectH = (rx,ry,w,h)
            pygame.draw.rect( self.screen, S.vlightBlue, rectH )
            if day < 10: tday = " " + str(day)
            else: tday = str(day)
            disputil.DrawText( self.screen, tday, self.fn, self.dateTh/2, 1, S.clockBkg, blit=True, x=rx+columnWidth, y=ry+columnWidth*1.7 )

            self.fn = disputil.InitFonts( S.font )                                      # Coming back to "general" font

            # Print rest of values: time and date
            disputil.DrawText( self.screen, tm0, self.fn, self.byTh, 0, S.loc, blit=True, x=xgap, y=ygap )
            disputil.DrawText( self.screen, tm1a, self.fn, self.dateTh/2, 0, S.clockc, blit=True, x=x, y=self.calYPos )
            disputil.DrawText( self.screen, tm1b, self.fn, self.dateTh, 0, S.clockc, blit=True, x=x, y=self.calYPos+ty1a )
            disputil.DrawText( self.screen, tm2, self.fn, self.tmTh2, 0, S.clockc, blit=True, x=xgap, y=self.tmdateYPos2 )
            disputil.DrawText( self.screen, tm3, self.fn, self.tmSTh2, 1, S.clockc, blit=True, x=xgap+tx2, y=self.tmdateYPos2+ty3*0.36)
            disputil.DrawText( self.screen, tm4, self.fn, int(self.dateTh*1.5), 1, S.clockc, blit=True, x=(self.xmax/2)-((tx4+tx5)/2), y=self.ymax-ty4-ygap )
            disputil.DrawText( self.screen, tm5, self.fn, self.dateTh, 0, S.clockc, blit=True, x=(self.xmax/2)-((tx4+tx5)/2)+tx4, y=self.ymax-ty5*1.1-ygap )

            pygame.display.update()
        else:
            if  int(tm3) == 0:
                rect = ( xgap, self.tmdateYPos2, tx2+tx3, ty2 )
                pygame.draw.rect( self.screen, S.clockBkg, rect )
                disputil.DrawText( self.screen, tm2, self.fn, self.tmTh2, 0, S.clockc, blit=True, x=xgap, y=self.tmdateYPos2 )
                disputil.DrawText( self.screen, tm3, self.fn, self.tmSTh2, 1, S.clockc, blit=True, x=xgap+tx2, y=self.tmdateYPos2+ty3*0.36)
            else:
                rect = ( xgap+tx2, self.tmdateYPos2+ty3*0.3, tx3, ty3 )
                pygame.draw.rect( self.screen, S.clockBkg, rect )
                disputil.DrawText( self.screen, tm3, self.fn, self.tmSTh2, 1, S.clockc, blit=True, x=xgap+tx2, y=self.tmdateYPos2+ty3*0.36)

            pygame.display.update( rect )
 
        return

    ####################################################################
    def time_date( self, line=True ):

        # Preparing data to display
        tm0 = "by alef"
        tm1 = time.strftime( "%b %d, ", time.localtime() )                          # Date
        tmx = wtranslat.elimina_tilde( time.strftime( "%A", time.localtime() ) )    # Day of week
        tm1 = tm1 + tmx
        tm1 = tm1.title()
        tm2 = time.strftime( "%H:%M", time.localtime() )                            # Hour:Minutes
        tm3 = time.strftime( "%S", time.localtime() )                               # Seconds
        tm4 = str.capitalize( S.wsource)                                            # Weather API
        tm5 = self.location                                                         # Location Name
        tm6 = wtranslat.TranslateT (S.lang, 106 ) + self.last                       # Last Update Time

        # Atenuate colors at Night
        if self.modeNight:
            self.dtc = S.dtcNight
            self.ccc = S.cccNight
        else:
            self.dtc = S.dtc
            self.ccc = S.ccc

        # Calculating size for relative positions
        (tx1,ty1) = disputil.DrawText( self.screen, tm1, self.fn, self.dateTh, 1, S.lc, blit=False )
        (tx2,ty2) = disputil.DrawText( self.screen, tm2, self.fn, self.tmTh, 1, S.dtc, blit=False )
        (tx3,ty3) = disputil.DrawText( self.screen, tm3, self.fn, self.tmSTh, 1, S.dtc, blit=False )
        (tx4,ty4) = disputil.DrawText( self.screen, tm3, self.fn, self.sourceTh, 1, S.loc, blit=False )
        (tx5,ty5) = disputil.DrawText( self.screen, tm4, self.fn, self.lSmTh, 0, S.lc, blit=False )
        (tx6,ty6) = disputil.DrawText( self.screen, tm4, self.fn, self.lSmTh, 0, S.lc, blit=False )

        # Erase previous values on screen
        rect = (self.xmin, self.ymin, self.xmax, ty2-self.ymargin)
        if S.tdBkg == None and S.showBkg:
            outline = 'outline'
            owidth = 3
            owidth1 = 2
        else:
            pygame.draw.rect( self.screen, S.tdBkg, rect )
            outline = None
            owidth = owidth1 = 0

        #Draw new values
        disputil.DrawText( self.screen, tm0, self.fn, self.sourceTh, 0, S.loc, blit=True, x=self.xmin+self.xmargin, y=self.ymin+self.ymargin*2, outline=outline, owidth=owidth1 )
        disputil.DrawText( self.screen, tm1, self.fn, self.dateTh, 1, S.lc, blit=True, x=self.xmin+self.xmargin, y=ty2-ty1-self.ymargin*2, outline=outline, owidth=owidth1 )
        disputil.DrawText( self.screen, tm2, self.fn, self.tmTh, 1, self.dtc, blit=True, x=self.xmax*S.txp-tx2/2, y=self.ymax*S.tmdateYPos, outline=outline, owidth=owidth )
        disputil.DrawText( self.screen, tm3, self.fn, self.tmSTh, 1, self.dtc, blit=True, x=self.xmax*S.txp+tx2/2, y=self.ymax*(S.tmdateYPos+S.tmdateYPosSm ), outline=outline, owidth=owidth )
        disputil.DrawText( self.screen, tm4, self.fn, self.sourceTh, 1, S.loc, blit=True, x=self.xmax*S.lp, y=ty2-ty4-ty5-ty6-self.ymargin*2, outline=outline, owidth=owidth1 )
        disputil.DrawText( self.screen, tm5, self.fn, self.lSmTh, 0, S.lc, blit=True, x=self.xmax*S.lp, y=ty2-ty5-ty6-self.ymargin*2, outline=outline, owidth=owidth1 )
        disputil.DrawText( self.screen, tm6, self.fn, self.lSmTh, 0, S.lc, blit=True, x=self.xmax*S.lp, y=ty2-ty6-self.ymargin*2, outline=outline, owidth=owidth1 )

        # Draw line and Update display (if specified)
        if line:
            if S.tdBkg == None:
                pygame.draw.line( self.screen, S.llc, (self.xmin+self.xmargin*3,ty2-self.ymargin),(self.xmax-self.xmargin*3,ty2-self.ymargin), S.lines )
            else:
                pygame.draw.line( self.screen, S.llc, (self.xmin,ty2-self.ymargin),(self.xmax,ty2-self.ymargin), S.lines )
        elif S.showBkg:
            disputil.dim( self.screen, S.dimFactor, S.black, (self.xmin, ty2-self.ymargin, self.xmax, self.ymargin*2) )

        self.headerHeight = ty2 - self.ymargin + line

        return

    ####################################################################
    def current_cond( self, icon, temp, temptext ):

        # dim only current conditions area (according to settings)
        if S.showBkg and (S.dimBkg == 'cc'):
            if ( ( int( self.temp ) > 9 ) or ( int( self.temp ) < 0 ) ): x = self.xmax * 0.4
            else: x = self.xmax * 0.45
            y = self.ymax*S.tp+self.ymargin
            w = self.xmax - x
            h = self.ymax - y - self.ymax * 0.42
            disputil.dim( self.screen, S.dimFactor, S.black, (x, y, w, h) )

        # Current conditions Icon (scaled)
        (ix,iy) = icon.get_size()
        icon = pygame.transform.scale( icon, (int(ix*self.iconScale),int(iy*self.iconScale)) )
        (ix,iy) = icon.get_size()
        self.screen.blit( icon, (self.xmax*S.ixp-ix*0.6,self.ymax*S.iyp) )

        # Outside Temp
        if not S.showBkg:
            if int(temp) > S.WarmTemp:
                if int(temp) > S.TooWarmTemp: self.ccc = S.cccTCW
                else: self.ccc = S.cccW
            elif int(temp) < S.ColdTemp:
                if int(temp) < S.TooColdTemp: self.ccc = S.cccTCW
                else: self.ccc = S.cccC
        (tx,ty) = disputil.DrawText( self.screen, temp, self.fn, self.outTempTh, 1, self.ccc, blit=False )
        disputil.DrawText( self.screen, temp, self.fn, self.outTempTh, 1, self.ccc, True, self.xmax/2-tx/2, self.ymax*S.tp, S.ol, 4 )
        disputil.DrawText( self.screen, S.uniTmp, self.fn, self.outTempDTh, 1, self.ccc, True, self.xmax/2+tx/2, self.ymax*S.dp, S.ol, 3)
        disputil.DrawText( self.screen, temptext, self.fn, self.outTempTTh, 1, S.lc, True, self.xmax/2-tx/2, self.ymax*S.tp+ty*0.9, S.ol, 2)

        return

    ####################################################################
    def other_cond( self, feels_like, wind_speed, wind_dir, baro, humid ):

        # Windchill
        condText = wtranslat.TranslateT( S.lang, 101 )
        disputil.DrawText( self.screen, condText, self.fn, self.regTh, 1, S.ctc, True, self.xmax*S.xp, self.ymax*S.st, S.ol, 2 )
        disputil.DrawText( self.screen, feels_like, self.fn, self.regTh, 1, S.cvc, True, self.xmax*S.x2,self.ymax*S.st, S.ol, 2 )

        # Wind
        condText = wtranslat.TranslateT( S.lang, 102)
        disputil.DrawText( self.screen, condText, self.fn, self.regTh, 1, S.ctc, True, self.xmax*S.xp,self.ymax*(S.st+S.gp2*1), S.ol, 2 )
        windStr = "%.0f %s" % ( wind_speed, S.windSpeed)
        if int( wind_speed ) > S.WindAlert: self.cws = S.cwsH
        else: self.cws = S.cws
        disputil.DrawText( self.screen, windStr, self.fn, self.regTh, 1, self.cws, True, self.xmax*S.x2,self.ymax*(S.st+S.gp2*1), S.ol, 2 )

        # Wind Direction
        condText = wtranslat.TranslateT( S.lang, 103 )
        disputil.DrawText( self.screen, condText, self.fn, self.regTh, 1, S.ctc, True, self.xmax*S.xp,self.ymax*(S.st+S.gp2*2), S.ol, 2 )
        disputil.DrawText( self.screen, wind_dir, self.fn, self.regTh, 1, S.cvc, True, self.xmax*S.x2, self.ymax*(S.st+S.gp2*2), S.ol, 2 )

        # Barometer/Pressure
        condText = wtranslat.TranslateT( S.lang, 104 )
        disputil.DrawText( self.screen, condText, self.fn, self.regTh, 1, S.ctc, True, self.xmax*S.xp,self.ymax*(S.st+S.gp2*3), S.ol, 2 )
        disputil.DrawText( self.screen, baro+S.baroUnits, self.fn, self.regTh, 1, S.cvc, True, self.xmax*S.x2,self.ymax*(S.st+S.gp2*3), S.ol, 2 )

        # Humidity
        condText = wtranslat.TranslateT( S.lang, 105 )
        disputil.DrawText( self.screen, condText, self.fn, self.regTh, 1, S.ctc, True, self.xmax*S.xp,self.ymax*(S.st+S.gp2*4), S.ol, 2 )
        disputil.DrawText( self.screen, humid+" %", self.fn, self.regTh, 1, S.cvc, True, self.xmax*S.x2,self.ymax*(S.st+S.gp2*4), S.ol, 2 )

        return

    ####################################################################
    def sub_win( self, subwin, day, icon, iconN, tempH, tempL, rain, line ):

        subwinCenter = self.xmax*S.wx*((subwin*2)+1)
        
        # Draw day name
        day = wtranslat.TranslateD( S.lang, day )
        (tx,ty) = disputil.DrawText( self.screen, day, self.fn, self.subwinTh, 1, S.lc, False )
        disputil.DrawText( self.screen, day, self.fn, self.subwinTh, 1, S.lc, True, subwinCenter-tx/2,self.ymax*S.wy, S.ol, 2 )

        # Draw icon
        icon = getIcon( icon )
        (ix,iy) = icon.get_size()
        if self.scaleIcon:
            icon = pygame.transform.scale( icon, (int(ix*self.iconScaleSub),int(iy*self.iconScaleSub)) )
            (ix,iy) = icon.get_size()
        self.screen.blit( icon, (subwinCenter-ix/2-self.xmargin,self.ymax*(S.wy+S.gp)) )
        if iconN != None and S.showNightf:
            icon = pygame.transform.scale( getIcon( iconN ), (ix/3,iy/3) )
            self.screen.blit( icon, (subwinCenter+ix/3,self.ymax*(S.wy+S.gp)) )

        # Draw High/Low temp and rain chance
        (htx,hty) = disputil.DrawText( self.screen, tempH, self.fn, self.subwinTh, 1, S.lc, blit=False, owidth=5 )
        (ltx,lty) = disputil.DrawText( self.screen, tempL, self.fn, self.subwinTh, 1, S.lc, blit=False, owidth=5 )
        (tx,ty) = (htx+ltx, hty)
        (rtx,rty) = disputil.DrawText( self.screen, rain, self.fn, self.subwinRTh, 1, S.lc, False )
        (ptx,pty) = disputil.DrawText( self.screen, "%", self.fn, self.subwinTh, 1, S.lc, False )

        if rain == "100": gap = self.xmargin
        else: gap = self.xmargin*2

        x = subwinCenter-(tx+rtx+ptx+gap)/2
        y = self.ymax-ty-self.ymargin*2
        disputil.DrawText( self.screen, tempH, self.fn, self.subwinTh, 1, S.lc, True, x, y, 'outrect', 5, S.fHTc, 128 )
        disputil.DrawText( self.screen, tempL, self.fn, self.subwinTh, 1, S.lc, True, x+htx, y, 'outrect', 5, S.fLTc, 128 )
        self.crc = S.crc
        if int(rain) > S.RainHigh: self.crc = S.crcw
        elif int(rain) > 0 : self.crc = S.crcm
        x = x + tx + gap
        disputil.DrawText( self.screen, rain, self.fn, self.subwinRTh, 1, self.crc, True, x, self.ymax-rty, S.ol, 3 )
        x = x + rtx
        disputil.DrawText( self.screen, "%", self.fn,  self.subwinTh, 1, self.crc, True, x, self.ymax-pty*1.2, S.ol, 2 )

        # Draw line
        if line: pygame.draw.line( self.screen, S.llc, (self.xmax/S.NSub*(subwin+1),self.ymax*(S.wy+0.03)),(self.xmax/S.NSub*(subwin+1),self.ymax), S.lines )

        return

    ####################################################################
    def DisplayWeather( self ):

        if S.showBkg:
            # Set background (maybe only suitable for large screens)
            if self.bkgCode != self.pbkgCode:
                self.pbkgCode = self.bkgCode
                self.bkg = getBkg( self.bkgCode )
                (bx, by) = self.bkg.get_size()
                # Scale bkg if size doesn't match your screen (mine are 1024x600)
                if (bx, by) != (self.xmax, self.ymax):
                    self.bkg = pygame.transform.scale( self.bkg, ( self.xmax, self.ymax ) )
            self.screen.blit( self.bkg, ( self.xmin, self.ymin, self.xmax, self.ymax ) )
            if  ( ( S.dimBkg == "all") or ( ( S.dimBkg == "light") and ( int( self.bkgCode ) in S.lightBkg ) ) ):
                disputil.dim( self.screen, S.dimFactor )
            if S.tdBkg == None and S.showBkg:
                tdRect = self.DisplayTime( blit=False )
                self.bkgPart = disputil.Screenshot( self.screen, (tdRect[0],tdRect[1]), (tdRect[2],tdRect[3]) )
        else:
            # Fill the screen with background color
            self.screen.fill( S.cBkg )

        # Header: Date, Time and Location
        self.time_date()

        # Current Conditions
        icon = getIcon( self.iconNow )
        self.current_cond( icon, self.temp, self.temptext)

        # Other conditions
        self.other_cond( self.feels_like, self.wind_speed, self.wind_dir, self.baro, self.humid )

        # SubWindows Forecasts
        for i in range (0, S.NSub):
            if i != (S.NSub-1): line = True
            else: line = False
            self.sub_win( i, self.day[i], self.icon[i], self.iconNight[i], self.temps[i][0], self.temps[i][1], self.rain[i], line )

        # Update the display
        pygame.display.update()

        return

    #####################################################################
    def CatchKbAction( self, key ):

        dispChanged = False

        # Check if keyboard has been pressed and the key is a different one
        if key != None and key != self.keyP:
            self.keyP = key
            # Check for Location Change (assigned to numbers)
            if key >= "1" and key <= str(len(S.zip_code)):
                try:
                    self.zip_code = S.zip_code[int(key)-1]
                    self.wLastUpdate = ''
                    if myDisp.UpdateWeather():
                        myDisp.DisplayWeather()
                        dispChanged = True
                except:
                    self.zip_code = S.zip_code[0]
                    print "NEW LOCATION %s: Error getting Weather information. Check ID in S.py or internet Connection" % S.zip_code[int(key)-1]
            # Other keys (Actions) could be placed below:


        return dispChanged

    #####################################################################
    def UpdateNews ( self ):

        titles = "No News data from " + self.nsource
        separator = "   |   "

        # Get news from RSS source and parse them into string variable
        try:
            with contextlib.closing(urllib.urlopen(self.nURL)) as response:
                if self.nsource == 'rtve': 
                    n = json.loads( response.read() )
                    titles = self.nsource + time.strftime( ' %H:%M') + separator
                    for i in range (0,4):
                        titles = titles + n['page']['items'][i]['longTitle'] + separator
                else:
                    n = ET.fromstring( response.read() )
                    titles = self.nsource + time.strftime( ' %H:%M') + separator
                    i = 0
                    for item in n.findall( './channel/item' ):
                        if i < 5:
                            titles = titles + item.find('title').text + separator
                            i = i+1
                        else: break
                    
        except KeyError:
            print "KeyError -> News Error"
        except ValueError:
            print "ValueError -> News Error"
        except:
            print "Error getting News from " + self.nsource

        # Shift News source for next update according to settings
        if S.alternSource:
            if self.nsource == S.nsource2:
                self.nsource = S.nsource1
                self.nURL = S.nURL1
            else:
                self.nsource = S.nsource2
                self.nURL = S.nURL2

        return titles

    ####################################################################
    def DisplayNews( self ):

        titles = myDisp.UpdateNews()

        rect = rectb = ( self.xmin, int(self.ymax*self.nPos), self.xmax, int(S.nSize*1.25) )
        if S.dimBkgNews:
            # Saving all the dimmed area to blit it later on (after showing the News)
            rectb = (self.xmin, self.headerHeight, self.xmax, self.ymax)
            img = disputil.Screenshot( self.screen, (rectb[0],rectb[1]), (rectb[2],rectb[3]) )
            # Dimming the background
            if  ( ( S.dimBkg == "all") or ( ( S.dimBkg == "light") and ( int( self.bkgCode ) in S.lightBkg ) ) ):
                disputil.dim( self.screen, S.dimFactor*2, S.black, rectb, True )
            else: disputil.dim( self.screen, S.dimFactor*3, S.black, rectb, True )
        else:
            # Saving the News area to blit it later on (after showing the News)
            img = disputil.Screenshot( self.screen, (rectb[0],rectb[1]), (rectb[2],rectb[3]) )

        spheres = ( scrolltx.ScrollText( self.screen, titles, self.ymax*self.nPos, S.nc, self.fn, S.nSize, S.nBold ), )
        #clock = pygame.time.Clock()

        changed = False
        t = None
        i = 0

        while ( i <= S.nTime ):
            #clock.tick(S.fps)
            s = int( time.strftime( "%S", time.localtime() ) )

            if ( s != t ):
                myDisp.DisplayTime()
                t = s
                i += 1

            pygame.draw.rect( self.screen, S.nBkg, rect )

            for thing in spheres:
                thing.update()

            pygame.display.update( rect )

            if myDisp.CatchKbAction(disputil.event_loop()):
                changed = True

            time.sleep(1.0/float(S.fps))

        if changed:
            # If the weather source has changed while showing News, update the whole screen (Screenshot will not be valid)
            myDisp.DisplayWeather()
        else:
            # Otherwise, update just the required area (News or dimmed background)
            self.screen.blit( img, (rectb[0],rectb[1]) )
            pygame.display.update( rectb )

        return

####################################################################

myDisp = MyDisplay()

# Loads data from Weather source into class variables, and display it
if myDisp.UpdateWeather( firstRun=True ):
    myDisp.DisplayWeather()
else:
    myDisp.DisplayOnlyTime( firstRun=True )

t = None            # Seconds control to update time, just once per second
running = True

while running:

    s = int( time.strftime( "%S", time.localtime() ) )
    m = None        # Minutes control to update/display News and Weather, but just once and at first run

    if ( t != s ):
        myDisp.DisplayTime()

        if ( (s == 0) or (t == None) ): m = int( time.strftime( "%M", time.localtime() ) )
        t = s

        if ( ( S.NewsMode == 'alwaysON' ) or ( ( S.NewsMode == 'period') and ( m in S.nPeriod ) ) ):
            myDisp.DisplayNews()
            
        if ( ( S.NewsMode == 'alwaysON' ) or ( m in S.wPeriod ) ):
            if myDisp.UpdateWeather():
                myDisp.DisplayWeather()

    myDisp.CatchKbAction(disputil.event_loop())

    # Reduce CPU consumption. Higher values produce unexpected results
    time.sleep( 0.2 )
