#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017 Angel Luis Escribano <alescribano@gmail.com>
################################################################
# WHAT IS THIS FOR?
#   - Shows Weather (current and forecasts) from Weather.com
#   - Latest News from RTVE or BBC 
# WARNING: Should you want other sources, you will have to adapt/struggle with the APIs
################################################################
# Based on the work of many people. Thank you for sharing!!!
#    WEATHER: Jim Kemp < https://www.instructableS.com/id/Raspberry-Pi-Internet-Weather-Station/ >
#    ICONS: VClouds http://vcloudS.deviantart.com/ 
#    BACKGROUNDS: Dr. Toxic and Timdog82001
#    NEWS TICKER (ScrollText): Gunny26 < https://github.com/gunny26/pygame/blob/master/ScrollText.py >
#    SCREEN dimMER: Thobias Thelen < http://www.pygame.org/pcr/screen_dimmer/index.php >
#    TFT GPIO SCREENS: wborelli < https://www.instructableS.com/id/Raspberry-Pi-Internet-Weather-Station/ > (NOT Tested)
################################################################
### BEGIN LICENSE
# See README.TXT File
### END LICENSE
################################################################

__version__ = "1.6.3"
# Clock version

import pygame
import time
import string
import urllib
import contextlib
import json
import xml.etree.ElementTree as ET
import calendar
import settingsD as S
import disputil
import wtranslat
import scrolltx

###############################################################################
def getIcon( code ):
    try:
        # Add .convert_alpha() if you find transparency troubles showing icon
        icon = pygame.image.load( S.iconf + str(code) + ".png" )
    except:
        icon = pygame.image.load( S.iconf + "na.png" )
    return icon

###############################################################################

class MyDisplay:
    screen = None;
    
    ####################################################################
    def __init__(self):

        if S.dispSize != None:
            # Change S.dispSize to try other resolutions (tested: 1920x1080, 1024x600, 800x600)
            # If S.dispSize matches screen size, it will turn into fullscreen mode and mouse will be hided
            self.screen, self.xmax, self.ymax = disputil.InitDisplay( S.dispSize[0], S.dispSize[1], windowed=True, icon=S.iconW, caption=S.caption)
        else:
            self.screen, self.xmax, self.ymax = disputil.InitDisplay( icon=S.iconW, caption=S.caption )
        self.xmin = self.ymin = 0
        self.xmargin = self.xmax*0.01
        self.ymargin = self.ymax*0.01
        self.xgap = self.xmargin*3
        self.ygap = self.ymargin*3

        # Icon scaling according to resolution and size (mine are optimized for 1024x600)
        self.scaleIconC = self.scaleIconF = True
        # Try this formula to calculate desired size (128 is the target for 1024x600, but will not always work!!!)
        #self.iconScaleCSub =  ( ( 128.0 * self.xmax ) / 1024.0 ) /  iconSize
        self.iconScaleC = self.xmax / float(S.optimdispSize[0])*1.5*S.iconScale     # Icon scale amount for main icon (current conditions)
        self.iconScaleF = self.xmax / float(S.optimdispSize[0])*0.8*S.iconScale     # Icon scale amount for sub icons (forecasts)

        # These values will change according to some conditionS. "Saving" them to self. variables
        self.nsource = S.nsource1
        self.nURL = S.nURL1
        self.zip_code = S.zip_code[0]
        self.keyP = None
        self.onlyTime = False
        self.errCount = 0
        self.sepPos = 0

        # Weather info initialization
        self.location = ''
        self.iconNow = 'na'
        self.bkgCode = 'na'
        self.temp = ''
        self.temptext = ''
        self.feels_like = '0'
        self.wind_speed = '0'
        self.baro = '29.95'
        self.wind_dir = 'S'
        self.humid = '50.0'
        self.wLastUpdate = ''
        self.last = ''
        self.day = [ '', '', '', '' ]
        self.icon = [ 0, 0, 0, 0 ]
        self.iconNight = [ 0, 0, 0, 0 ]
        self.rain = [ '', '', '', '' ]
        self.temps = [ ['',''], ['',''], ['',''], ['',''] ]
        self.sunrise = '7:00 AM'
        self.sunset = '8:00 PM'

        # Fonts initialization
        self.fn = disputil.InitFonts( S.font )              # Standard font
        self.fncal = disputil.InitFonts( S.calfont )        # Calendar font ("mono", non-proportional)
        self.fntime = disputil.InitFonts( S.timefont )      # Time and date font (extralight)

        # Text Sizes
        self.byTh = int(self.ymax*S.byTh)                   # "by" text Height
        self.yearTh = int(self.ymax*S.yearTh)               # Year Text Height
        self.wdayTh = int(self.ymax*S.wdayTh)               # Day of week Text Height
        self.mdayTh = int(self.ymax*S.mdayTh)               # Day of month and month Text Height
        self.condTh = int(self.ymax*S.condTh)               # Regular Font
        self.timeTh = int(self.ymax*S.timeTh)               # Time Text Height (Hours:Minutes)
        self.secTh = int(self.ymax*S.secTh)                 # Seconds Text Height
        self.cityTh = int(self.ymax*S.cityTh)               # Location and update Text Height
        self.statTh = int(self.ymax*S.statTh)               # Source text Height
        self.tempTh = int(self.ymax*S.tempTh)               # Outside temperature Text Height
        self.degTh = int(self.ymax*S.degTh)                 # Degrees symbol for outside temperature Text Height
        self.temptxTh = int(self.ymax*S.temptxTh)           # Outside conditions text height
        self.subtempTh = int(self.ymax*S.subtempTh)         # Sub window text height
        self.subrainTh = int(self.ymax*S.subrainTh)         # Rain chance text height
        self.calTh = int(self.ymax*S.calTh)                 # Calendar Text Height

        # Positions
        self.clockYPos= self.ymax*S.clockYPos               # Clock position
        self.clockSYPos = self.clockYPos+self.secTh*0.2     # Seconds position
        self.calYPos = self.ymax*S.calYPos                  # Calendar position
        self.subXPos = self.xmax*S.subXPos                  # SubWindows (forecasts) Xaxis start position
        self.subYPos = self.ymax*S.subYPos                  # SubWindows (forecasts) Yaxis start position

    ####################################################################
    def __del__(self):
        "Destructor to make sure pygame shuts down, etc."

    ####################################################################
    def ParseWeathercom( self, w ):

        wUpdated = False

        if ( w[S.cc][0].text != self.wLastUpdate ):
            wUpdated = True
            self.last = time.strftime( '%H:%M')

            # Current Conditions
            self.wLastUpdate = w[S.cc][0].text
            self.location = w[S.cc][1].text
            self.temp = w[S.cc][2].text
            self.feels_like = w[S.cc][3].text + S.degree_sign
            self.iconNow = w[S.cc][5].text
            if self.iconNow == None: self.iconNow = w[S.f][1][4][0].text
            if self.iconNow == None: self.iconNow = w[S.f][1][5][0].text
            self.bkgCode = self.iconNow
            if S.lang  == 'en': self.temptext = string.capitalize( w[S.cc][4].text )
            else: self.temptext = string.capitalize( wtranslat.TranslateT( S.lang, int( self.iconNow ) ) )
            self.baro = w[S.cc][6][0].text
            self.wind_speed = string.upper ( w[S.cc][7][0].text )
            if 'CALM' in self.wind_speed: self.wind_speed = 0
            else: self.wind_speed = int(self.wind_speed) * S.windScale
            self.wind_dir = string.upper( w[S.cc][7][3].text )
            if 'VAR' in self.wind_dir: self.wind_dir = 'Var'
            if 'CALM' in self.wind_dir: self.wind_dir = '--'
            if S.lang == 'es': self.wind_dir = self.wind_dir.replace( "W", "O" )
            self.humid = w[S.cc][8].text
            self.sunrise = w[1][4].text
            self.sunset = w[1][5].text
            if ( self.sunrise[1] == ":" ): self.sunrise = "0" + self.sunrise
            self.sr = int( self.sunrise[:2] )
            if ( self.sunset[1] == ":" ): self.sunset = "0" + self.sunset
            self.sn = int( self.sunset[:2] ) + 12

            # Forecasts
            i = 0
            for day in w.findall('./dayf/day'):
                self.day[i] = wtranslat.TranslateD( S.lang, w[S.f][i+1].get('t') )
                self.icon[i] = w[S.f][i+1][4][0].text
                self.iconNight[i] = w[S.f][i+1][5][0].text
                self.rain[i] = w[S.f][i+1][4][4].text
                # add  + S.degree_sign, if you want the symbol back!
                self.temps[i][0] = w[S.f][i+1][0].text
                self.temps[i][1] = w[S.f][i+1][1].text
                if i == 0:
                    self.day[i] = wtranslat.TranslateD( S.lang, "Today" )
                    # At some moment, some info for Today is null, taking "Night part"
                    if self.icon[i] == None:
                        self.icon[i] = w[S.f][i+1][5][0].text
                        self.iconNight[i] = None
                        self.rain[i] = w[S.f][i+1][5][4].text
                i += 1

            hh = int(self.last[:2])
            if ( ( ( hh > 12 ) and ( hh > int(self.sn) ) ) or ( ( hh < 12 ) and ( hh < self.sr ) ) ):
                self.modeNight = True
            else: self.modeNight = False

        return wUpdated

    ####################################################################
    def ParseYahooWeather( self, w ):

        wUpdated = False

        if ( w[S.qu][S.re][S.ch][S.it]['pubDate'] != self.wLastUpdate ):
            wUpdated = True
            self.last = time.strftime( '%H:%M')

            # Current Conditions
            self.wLastUpdate = w[S.qu][S.re][S.ch][S.it]['pubDate']
            self.location = w[S.qu][S.re][S.ch]['title']
            self.location = self.location[17:]
            self.iconNow = w[S.qu][S.re][S.ch][S.it][S.cc]['code']
            self.bkgCode = self.iconNow
            self.temp = w[S.qu][S.re][S.ch][S.it][S.cc]['temp']
            if S.lang  == 'es': self.temptext = string.capitalize( wtranslat.TranslateT( S.lang, int( self.iconNow ) ) )
            else: self.temptext = string.capitalize( w[S.qu][S.re][S.ch][S.it][S.cc]['text'] )
            self.feels_like = w[S.qu][S.re][S.ch]['wind']['chill'] + S.degree_sign
            if S.disp_units == 'metric': self.feels_like = str( int ( ( int( self.feels_like[:-1] ) - 32 ) / 1.8 ) ) + S.degree_sign
            self.wind_speed = float( w[S.qu][S.re][S.ch]['wind']['speed'] ) * S.windScale
            self.wind_dir = wtranslat.TranslateWindDir( float( w[S.qu][S.re][S.ch]['wind']['direction'] ) )
            if S.lang == 'es': self.wind_dir = self.wind_dir.replace( "W", "O" )
            self.baro = w[S.qu][S.re][S.ch]['atmosphere']['pressure']
            if S.disp_units == "metric": self.baro = str( round( float( self.baro ) / S.baroScale, 2 ) )
            self.humid = w[S.qu][S.re][S.ch]['atmosphere']['humidity']
            self.sunrise = w[S.qu][S.re][S.ch]['astronomy']['sunrise'] 
            self.sunset = w[S.qu][S.re][S.ch]['astronomy']['sunset']
            if ( self.sunrise[1] == ":" ): self.sunrise = "0" + self.sunrise
            self.sr = int( self.sunrise[:2] )
            if ( self.sunset[1] == ":" ): self.sunset = "0" + self.sunset
            self.sn = int( self.sunset[:2] ) + 12

            # Forecasts
            for i in range (0,4):
                if i == 0: self.day[i] = wtranslat.TranslateD( S.lang, "Today" )
                else: self.day[i] = w[S.qu][S.re][S.ch][S.it][S.f][i]['day']
                self.icon[i] = w[S.qu][S.re][S.ch][S.it][S.f][i]['code']
                self.iconNight[i] = None
                self.rain[i] = str( 0 )  # No precipitation probability in API :(
                # add  + S.degree_sign, if you want the symbol back
                self.temps[i][0] = w[S.qu][S.re][S.ch][S.it][S.f][i]['high']
                self.temps[i][1] = w[S.qu][S.re][S.ch][S.it][S.f][i]['low']

            hh = int(self.last[:2])
            if ( ( ( hh > 12 ) and ( hh > int(self.sn) ) ) or ( ( hh < 12 ) and ( hh < self.sr ) ) ):
                self.modeNight = True
            else: self.modeNight = False

        return wUpdated

    ####################################################################
    def UpdateWeather( self, firstRun=False ):

        wUpdated = False
        
        # Get Weather information from source
        url = S.weatherURL % ( self.zip_code, S.query_units )
        try:
            with contextlib.closing(urllib.urlopen(url)) as response:
                if S.wsource == "Yahoo!":
                    w = json.loads( response.read() )
                    wUpdated = self.ParseYahooWeather( w )
                else:
                    w = ET.fromstring( response.read() )
                    wUpdated = self.ParseWeathercom( w )
            if self.onlyTime:
                self.onlyTime = False
                # This will force weather info to be drawn though it didn't change since last update
                wUpdated = True
            self.errCount = 0

        except KeyError:
            print "KeyError -> Weather Error"
        except ValueError:
            print "ValueError -> Weather Error"
        except:
            print "Error getting update from " + S.wsource + " " + self.last
            self.errCount += 1
            if firstRun or self.errCount > 12:
                # No Weather info or obsolete (show clock only)
                self.onlyTime = True
                # This will erase obsolete Weather info
                wUpdated = True

        return wUpdated

    ####################################################################
    def DisplayClock( self ):

        rect = (self.xmin, self.ymin, self.xmax, self.subYPos)
        pygame.draw.rect( self.screen, S.cBkg, rect )
        if S.showBkg:
            self.bkg = pygame.image.load( S.Bkg )
            if self.bkg.get_size() != S.dispSize:
                self.bkg = pygame.transform.scale( self.bkg, ( self.xmax, self.ymax ) )
            self.screen.blit( self.bkg, ( self.xmin, self.ymin, self.xmax, self.ymax ) )

        tm0 = "by alef"
        tm1a = wtranslat.elimina_tilde( time.strftime( "%A", time.localtime() ) )   # Day of week
        tm1b = time.strftime( "%B %d", time.localtime() )                           # Date
        tm4 = time.strftime( "%Y, ", time.localtime() )                             # Year

        (tx0,ty0) = disputil.DrawText( self.screen, tm0, self.fn, self.byTh, 0, S.byc, blit=False )
        (tx1a,ty1a) = disputil.DrawText( self.screen, tm1a, self.fn, self.wdayTh, 0, S.clockc, blit=False )
        (tx1b,ty1b) = disputil.DrawText( self.screen, tm1b, self.fn, self.mdayTh, 0, S.clockc, blit=False )
        (tx4,ty4) = disputil.DrawText( self.screen, tm4, self.fn, self.yearTh, 0, S.clockc, blit=False )
        y = self.calYPos + ty1a + ty1b

        # Calendar
        x = self.DisplayCalendar( y )

        # Draw rest of values: time and date
        self.DisplayTime( force=True )
        disputil.DrawText( self.screen, tm0, self.fn, self.byTh, 0, S.byc, blit=True, x=self.xgap, y=self.ygap )
        disputil.DrawText( self.screen, tm1a, self.fn, self.wdayTh, 0, S.clockc, blit=True, x=x, y=self.calYPos )
        disputil.DrawText( self.screen, tm1b, self.fn, self.mdayTh, 0, S.clockc, blit=True, x=x, y=self.calYPos+ty1a*0.9 )
        #disputil.DrawText( self.screen, tm4, self.fn, self.yearTh, 0, S.clockc, blit=True, x=x-tx4, y=self.calYPos )

        if self.location != '': self.DispLocation()
        
        pygame.display.update( rect )
 
        return

    ####################################################################
    def DisplayTime( self, force=False ):

        h = time.strftime( "%H", time.localtime() )                     # Hour
        sep = ":"
        m = time.strftime( "%M", time.localtime() )                     # Minutes
        s = time.strftime( "%S", time.localtime() )                     # Seconds
            
        if ( int(h)+int(m)+int(s) == 0 ) and not force:
            # Display entire clock at 00:00, but not if invoked from DisplayClock() (avoid circular calls)
            self.DisplayClock()
                
        elif ( int(s) == 0 ) or force:
            # Display all Time info when minutes (and hours) change, and at first run (force)
            
            (txh,tyh) = disputil.DrawText( self.screen, h, self.fntime, self.timeTh, 0, S.clockc, blit=False )
            (txsep,tysep) = disputil.DrawText( self.screen, sep, self.fntime, self.timeTh, 0, S.clockc, blit=False )
            (txm,tym) = disputil.DrawText( self.screen, m, self.fntime, self.timeTh, 0, S.clockc, blit=False )
            rect = ( self.xgap, self.clockYPos+tyh*0.15, txh+txsep+txm, tyh*0.7 )
            pygame.draw.rect( self.screen, S.cBkg, rect )

            disputil.DrawText( self.screen, h, self.fntime, self.timeTh, 0, S.clockc, blit=True, x=self.xgap, y=self.clockYPos )
            disputil.DrawText( self.screen, sep, self.fntime, self.timeTh, 0, S.clockc, blit=True, x=self.xgap+txh, y=self.clockYPos )
            disputil.DrawText( self.screen, m, self.fntime, self.timeTh, 0, S.clockc, blit=True, x=self.xgap+txh+txsep, y=self.clockYPos )

            self.sepPos = self.xgap + txh

            if not force: pygame.display.update( rect )

        else:
            # Display "blinking" separator (":") only every second

            if ( int(s) % 2 == 0): sepColor = S.clockc
            else: sepColor = S.clockh

            (txsep,tysep) = disputil.DrawText( self.screen, sep, self.fntime, self.timeTh, 0, sepColor, blit=True, x=self.sepPos, y=self.clockYPos )
            rect = (self.sepPos, self.clockYPos, txsep, tysep)

            pygame.display.update( rect )
 
        return

    ####################################################################
    def DisplayCalendar( self, YPos ):

        columnWidth = 3

        c = calendar.TextCalendar()                                                 # Calendar
        year = int( time.strftime( "%Y", time.localtime() ) )
        month = int( time.strftime("%m", time.localtime()) )
        day = int(time.strftime( "%d", time.localtime()) )
        cal = c.formatmonth( year, month, w=columnWidth ).splitlines()              # Calendar splitted into separate lines

        if S.lang == 'es': cal[1] = wtranslat.TranslateT( S.lang, 108 )             # Setting spanish header (troubles with accents)
        (tx,ty) = disputil.DrawText( self.screen, cal[1], self.fncal, self.calTh, 1, S.clockc, blit=False )
        x = self.xmax - tx - self.xgap
        y = YPos

        # Draw calendar
        for i in range (1,len(cal)):
            if i == 1: color = S.clockh
            else: color = S.clockc
            disputil.DrawText( self.screen, cal[i], self.fncal, self.calTh, 1, color, blit=True, x=x, y=y)
            y = y + ty + self.ygap
            
        # Highlight current day
        (leapDay,daysTotal) = calendar.monthrange( year, month )
        # Find column
        column = day % 7 + leapDay
        if column > 7: column = column - 7
        # Find row
        row = ((day + leapDay) / 7)
        if (day + leapDay) % 7 != 0: row = row + 1
        # Highlight
        rx = x + ( column * tx / 7 ) - self.xgap - columnWidth + column
        ry = YPos + ( row * ( ty + self.ygap ) ) - self.ymargin
        w = h = ty + columnWidth * 3
        rectH = (rx,ry,w,h)
        pygame.draw.rect( self.screen, S.clockc, rectH )
        if day < 10: tday = " " + str(day)
        else: tday = str(day)
        disputil.DrawText( self.screen, tday, self.fncal, self.calTh, 1, S.cBkg, blit=True, x=rx+columnWidth, y=ry+self.ymargin )

        return x

    ####################################################################
    def DispLocation( self ):

        if "/" in self.location:
            city = self.location.split("/")[0]
            station = "  " + self.location.split("/")[1]
        else:
            city = self.location.split(",")[0]
            if city == "Cuidad Real": city = "Ciudad Real"
            station = "  " + self.location.split(",")[1]+","+self.location.split(",")[2]
        station = station + " - " + self.last

        (tx,ty) = disputil.DrawText( self.screen, city, self.fn, self.cityTh, 1, S.clockc, blit=False )
        (tx1,ty1) = disputil.DrawText( self.screen, station, self.fn, self.statTh, 0, S.clockc, blit=False )

        rect = (self.xgap, self.subYPos-ty, self.xgap+tx+tx1, ty)
        pygame.draw.rect( self.screen, S.cBkg, rect )

        disputil.DrawText( self.screen, city, self.fn, self.cityTh, 1, S.clockc, blit=True, x=self.xgap, y=self.subYPos-ty )
        disputil.DrawText( self.screen, station, self.fn, self.statTh, 0, S.clockc, blit=True, x=self.xgap+tx, y=self.subYPos-ty1*1.07 )

        return rect
        
    ####################################################################
    def DispCurrentC( self, icon, temp, temptext ):

        if S.iconSet == 'A':     # This icon set is wider, and needs less margin
            x = 0
        else:
            x = self.xmargin*2

        # Current conditions Icon
        iconImg = getIcon( icon )
        (ix,iy) = iconImg.get_size()
        if self.scaleIconC:
            iconImg = pygame.transform.scale( iconImg, (int(ix*self.iconScaleC),int(iy*self.iconScaleC)) )
            (ix,iy) = iconImg.get_size()
        y = self.subYPos
        self.screen.blit( iconImg, (x,self.subYPos) )

        # Outside Temp
        (tx,ty) = disputil.DrawText( self.screen, temp, self.fn, self.tempTh, 1, S.wc, blit=False )
        (dtx,dty) = disputil.DrawText( self.screen, S.uniTmp, self.fn, self.degTh, 1, S.wc, blit=False )
        (ttx,tty) = disputil.DrawText( self.screen, temptext, self.fn, self.temptxTh, 1, S.wc, blit=False )
        
        disputil.DrawText( self.screen, temp, self.fn, self.tempTh, 1, S.wc, True, x+ix+self.xmargin, y+self.ymargin*2 )
        disputil.DrawText( self.screen, S.uniTmp, self.fn, self.degTh, 1, S.wc, True, x+ix+self.xmargin+tx, y+self.ymargin*2+ty*0.1 )
        disputil.DrawText( self.screen, temptext, self.fn, self.temptxTh, 1, S.wc, True, x+ix+self.xmargin, self.subYPos+ty )

        if len(temp) >  1:
            gap = self.xgap
        else: gap = self.xgap * 2
        XPos = x+ix+tx+dtx+gap
        return XPos, self.subYPos+self.ygap+self.ymargin

    ####################################################################
    def DispOtherC( self, XPos, YPos, feels_like, wind_speed, wind_dir, baro, humid ):

        # Windchill
        (tx,ty) = disputil.DrawText( self.screen, feels_like, self.fn, self.condTh, 1, S.wc, True, XPos, YPos )

        # Wind
        windStr = "%.0f %s" % ( wind_speed, S.windSpeed)
        disputil.DrawText( self.screen, windStr, self.fn, self.condTh, 1, S.wc, True, XPos, YPos+ty+self.ymargin )

        # Wind Direction
        disputil.DrawText( self.screen, wind_dir, self.fn, self.condTh, 1, S.wc, True, XPos, YPos+(ty+self.ymargin)*2 )

        # Barometer/Pressure
        disputil.DrawText( self.screen, baro+S.baroUnits, self.fn, self.condTh, 1, S.wc, True, XPos, YPos+(ty+self.ymargin)*3 )

        # Humidity
        disputil.DrawText( self.screen, humid+" %", self.fn, self.condTh, 1, S.wc, True, XPos, YPos+(ty+self.ymargin)*4 )

        return

    ####################################################################
    def DispForecasts( self, subwin, day, icon, iconN, tempH, tempL, rain, line ):

        subwinWidth = (self.xmax-self.subXPos)/S.NSub
        subwinCenter = self.subXPos + (subwinWidth*(subwin+1)) - subwinWidth/2

        # Draw icon
        iconImg = getIcon( icon )
        (ix,iy) = iconImg.get_size()
        if  self.scaleIconF:
            iconImg = pygame.transform.scale( iconImg, (int(ix*self.iconScaleF),int(iy*self.iconScaleF)) )
            (ix,iy) = iconImg.get_size()
        self.screen.blit( iconImg, (subwinCenter - ix/2, self.subYPos) )

        # Draw High/Low temp
        temp = tempH + " / " + tempL
        (tx,ty) = disputil.DrawText( self.screen, temp, self.fn, self.subtempTh, 0, S.clockc, blit=False )
        disputil.DrawText( self.screen, temp, self.fn, self.subtempTh, 0, S.wc, True, subwinCenter-tx/2, self.subYPos+iy )

        # Draw rain chance
        self.crc = S.clockc
        (rtx,rty) = disputil.DrawText( self.screen, rain, self.fn, self.subrainTh, 1, self.crc, False )
        (ptx,pty) = disputil.DrawText( self.screen, "%", self.fn, int(self.subrainTh/2), 1, self.crc, False )
        if int(rain) > S.RainHigh: self.crc = S.crcw
        elif int(rain) > 10: self.crc = S.crcm
        disputil.DrawText( self.screen, rain, self.fn, self.subrainTh, 1, self.crc, True, subwinCenter-rtx/2, self.subYPos+iy+ty )
        disputil.DrawText( self.screen, "%", self.fn, int(self.subrainTh/2), 1, self.crc, True, subwinCenter+rtx/2, self.subYPos+iy+ty+rty/2.2 )

        return

    ####################################################################
    def DisplayWeather( self ):

        rect = []
        rect.append( (self.xmin,self.subYPos,self.xmax,self.ymax) )
        pygame.draw.rect( self.screen, S.wBkg, rect[0] )

        if not self.onlyTime:

            # Location
            rect.append( self.DispLocation() )

            # Current Conditions
            (ccXPos,ccYPos) = self.DispCurrentC( self.iconNow, self.temp, self.temptext)

            # Other conditions
            self.DispOtherC( ccXPos, ccYPos, self.feels_like, self.wind_speed, self.wind_dir, self.baro, self.humid )

            # SubWindows Forecasts
            for i in range (0, S.NSub):
                if i != (S.NSub-1): line = True
                else: line = False
                self.DispForecasts( i, self.day[i], self.icon[i], self.iconNight[i], self.temps[i][0], self.temps[i][1], self.rain[i], line )

        # Update the display
        pygame.display.update( rect )

        return

    #####################################################################
    def CatchKbAction( self, key ):

        dispChanged = False

        # Check if keyboard has been pressed and the key is a different one
        if key != None and key != self.keyP:
            self.keyP = key
            # Check for Location Change (assigned to numbers)
            if key >= "1" and key <= str(len(S.zip_code)):
                try:
                    self.zip_code = S.zip_code[int(key)-1]
                    self.wLastUpdate = ''
                    if myDisp.UpdateWeather():
                        myDisp.DisplayWeather()
                        dispChanged = True
                except:
                    self.zip_code = S.zip_code[0]
                    print "NEW LOCATION %s: Error getting Weather information. Check ID in Settins.py or internet Connection" % S.zip_code[int(key)-1]
            # Other keys (Actions) could be placed below:


        return dispChanged

    #####################################################################
    def UpdateNews ( self ):

        self.titles = ""
        nUpdated = True

        # Get news from RSS source and parse them into string variable
        try:
            with contextlib.closing(urllib.urlopen(self.nURL)) as response:
                if self.nsource == 'rtve': 
                    n = json.loads( response.read() )
                    self.titles = self.nsource + time.strftime( ' %H:%M') + S.separator
                    for i in range (0,4):
                        self.titles = self.titles + n['page']['items'][i]['longTitle'] + S.separator
                else:
                    n = ET.fromstring( response.read() )
                    self.titles = self.nsource + time.strftime( ' %H:%M') + S.separator
                    i = 0
                    for item in n.findall( './channel/item' ):
                        if i < 5:
                            self.titles = self.titles + item.find('title').text + S.separator
                            i = i+1
                        else: break
                    
        except KeyError:
            print "KeyError -> News Error"
        except ValueError:
            print "ValueError -> News Error"
        except:
            print "Error getting News from " + self.nsource

        if self.titles == "":
            nUpdated = False

        # Shift News source for next update according to settings
        if S.alternSource:
            if self.nsource == S.nsource2:
                self.nsource = S.nsource1
                self.nURL = S.nURL1
            else:
                self.nsource = S.nsource2
                self.nURL = S.nURL2

        return nUpdated

    ####################################################################
    def DisplayNews( self ):

        (x,y,w,h) = ( self.xmin, int(self.ymax*S.nPos), self.xmax, int(S.nSize*1.25) )
        # Save display area which will be overwritten by the News ticker
        img = disputil.Screenshot( self.screen, (x,y), (w,h) )

        spheres = ( scrolltx.ScrollText( self.screen, self.titles, self.ymax*S.nPos, S.nc, self.fn, S.nSize, S.nBold ), )
        #clock = pygame.time.Clock()

        changed = False
        t = None
        i = 1

        while ( i <= S.nTime ):
            #clock.tick(S.fps)
            s = int( time.strftime( "%S", time.localtime() ) )

            if ( s != t ):
                myDisp.DisplayTime()
                t = s
                i += 1

            pygame.draw.rect( self.screen, S.nBkg, (x,y,w,h) )

            for thing in spheres:
                thing.update()

            pygame.display.update( (x,y,w,h) )

            if myDisp.CatchKbAction(disputil.event_loop()):
                changed = True
                
            time.sleep(1.0/float(S.fps))

        # Display previously saved area on News
        self.screen.blit( img, (x,y) )
        if changed:
            # If weather location changed while showing News, update it
            self.DispLocation()
        pygame.display.update( (x,y,w,h) )

        return

####################################################################

myDisp = MyDisplay()

# Display clock and calendar
myDisp.DisplayClock()

# Download and Display weather information
if myDisp.UpdateWeather( firstRun=True ):
    myDisp.DisplayWeather()

t = None            # Seconds control to update time, just once per second

while True:

    s = int( time.strftime( "%S", time.localtime() ) )
    m = None        # Minutes control to update/display News and Weather, but just once and at first run

    if ( t != s ):
        myDisp.DisplayTime()

        if ( (s == 0) or (t == None) ): m = int( time.strftime( "%M", time.localtime() ) )
        t = s

        if ( ( S.NewsMode == 'alwaysON' ) or ( ( S.NewsMode == 'period') and ( m in S.nPeriod ) ) ):
            if myDisp.UpdateNews():
                myDisp.DisplayNews()
            
        if ( ( S.NewsMode == 'alwaysON' ) or ( m in S.wPeriod ) ):
            if myDisp.UpdateWeather():
                myDisp.DisplayWeather()

    myDisp.CatchKbAction(disputil.event_loop())

    # Reduce CPU consumption. Higher values produce unexpected results
    time.sleep( 0.2 )
